@@ -42,6 +42,10 @@ use std::borrow::Cow;
 #[cfg(test)]
 use quickcheck::{Arbitrary, Gen};
 
+use base64::Engine;
+use base64::engine::general_purpose::STANDARD as base64std;
+use base64::engine::general_purpose::STANDARD_NO_PAD as base64nopad;
+
 use crate::packet::prelude::*;
 use crate::packet::header::{BodyLength, CTBNew, CTBOld};
 use crate::parse::Cookie;
@@ -377,8 +381,7 @@ impl<W: Write> Writer<W> {
 
         // Write any stashed bytes and pad.
         if !self.stash.is_empty() {
-            self.sink.write_all(base64::encode_config(
-                &self.stash, base64::STANDARD).as_bytes())?;
+            self.sink.write_all(base64std.encode(&self.stash).as_bytes())?;
             self.column += 4;
         }
 
@@ -404,7 +407,7 @@ impl<W: Write> Writer<W> {
 
         // CRC and footer.
         write!(self.sink, "={}{}{}{}",
-               base64::encode_config(&bytes, base64::STANDARD_NO_PAD),
+               base64nopad.encode(&bytes),
                LINE_ENDING, self.kind.end(), LINE_ENDING)?;
 
         self.dirty = false;
@@ -454,8 +457,7 @@ impl<W: Write> Write for Writer<W> {
             // If this fails for some reason, and the caller retries
             // the write, we might end up with a stash of size 3.
             self.sink
-                .write_all(base64::encode_config(
-                    &self.stash, base64::STANDARD_NO_PAD).as_bytes())?;
+                .write_all(base64nopad.encode(&self.stash).as_bytes())?;
             self.column += 4;
             self.linebreak()?;
             crate::vec_truncate(&mut self.stash, 0);
@@ -472,9 +474,9 @@ impl<W: Write> Write for Writer<W> {
             }
 
             written += input_bytes;
-            base64::encode_config_slice(&input[..input_bytes],
-                                        base64::STANDARD_NO_PAD,
-                                        &mut self.scratch[..encoded_bytes]);
+            base64nopad.encode_slice(&input[..input_bytes],
+                                  &mut self.scratch[..encoded_bytes])
+                .expect("buffer correctly sized");
 
             let mut n = 0;
             while ! self.scratch[n..encoded_bytes].is_empty() {
@@ -581,7 +583,7 @@ pub struct Reader<'a> {
 assert_send_and_sync!(Reader<'_>);
 
 // The default buffer size.
-const DEFAULT_BUF_SIZE: usize = 8 * 1024;
+const DEFAULT_BUF_SIZE: usize = 32 * 1024;
 
 impl<'a> fmt::Display for Reader<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -846,12 +848,14 @@ impl<'a> Reader<'a> {
                     let mut o = [ 0u8; 4 ];
 
                     CTBNew::new(tag).serialize_into(&mut ctb[..]).unwrap();
-                    base64::encode_config_slice(&ctb[..], base64::STANDARD, &mut o[..]);
+                    base64std.encode_slice(&ctb[..], &mut o[..])
+                        .expect("buffer correctly sized");
                     valid_start.push(o[0]);
 
                     CTBOld::new(tag, BodyLength::Full(0)).unwrap()
                         .serialize_into(&mut ctb[..]).unwrap();
-                    base64::encode_config_slice(&ctb[..], base64::STANDARD, &mut o[..]);
+                    base64std.encode_slice(&ctb[..], &mut o[..])
+                        .expect("buffer correctly sized");
                     valid_start.push(o[0]);
                 }
 
@@ -1241,8 +1245,7 @@ impl<'a> Reader<'a> {
                 // (Note: the computed size *might* be a slight
                 // overestimate, because the last base64 chunk may
                 // include padding.)
-                self.decode_buffer = base64::decode_config(
-                    &base64data, base64::STANDARD)
+                self.decode_buffer = base64std.decode(&base64data)
                     .map_err(|e| Error::new(ErrorKind::InvalidData, e))?;
 
                 let copied = cmp::min(buf.len(), self.decode_buffer.len());
@@ -1253,8 +1256,7 @@ impl<'a> Reader<'a> {
             } else {
                 // We can decode directly into the caller-supplied
                 // buffer.
-                base64::decode_config_slice(
-                    &base64data, base64::STANDARD, buf)
+                base64std.decode_slice(&base64data, buf)
                     .map_err(|e| Error::new(ErrorKind::InvalidData, e))?
             };
 
@@ -1534,12 +1536,6 @@ impl<'a> Reader<'a> {
            self.cursor,
            self.buffer.as_ref().map(|buffer| buffer.len()));
 
-        // See if there is an error from the last invocation.
-        if let Some(e) = self.error.take() {
-            t!("Returning stashed error: {}", e);
-            return Err(e);
-        }
-
         if let Some(ref buffer) = self.buffer {
             // We have a buffer.  Make sure `cursor` is sane.
             assert!(self.cursor <= buffer.len());
@@ -1554,9 +1550,9 @@ impl<'a> Reader<'a> {
             // The caller wants more data than we have readily
             // available.  Read some more.
 
-            let capacity : usize = cmp::max(cmp::max(
-                DEFAULT_BUF_SIZE,
-                2 * self.preferred_chunk_size), amount);
+            let capacity : usize = amount.saturating_add(
+                DEFAULT_BUF_SIZE.max(
+                    self.preferred_chunk_size.saturating_mul(2)));
 
             let mut buffer_new = self.unused_buffer.take()
                 .map(|mut v| {
@@ -1575,6 +1571,12 @@ impl<'a> Reader<'a> {
                     break;
                 }
 
+                // See if there is an error from the last invocation.
+                if let Some(e) = &self.error {
+                    t!("We have a stashed error, don't poll again: {}", e);
+                    break;
+                }
+
                 match self.do_read(&mut buffer_new
                                    [amount_buffered + amount_read..]) {
                     Ok(read) => {
@@ -2293,7 +2295,8 @@ mod test {
             types::HashAlgorithm,
         };
 
-        fn f<R>(clearsig: &[u8], reference: R) -> crate::Result<()>
+        fn f<R>(clearsig: &[u8], reference: R, hash: HashAlgorithm)
+                -> crate::Result<()>
         where R: AsRef<[u8]>
         {
             let mut reader = Reader::from_buffered_reader_csft(
@@ -2309,7 +2312,7 @@ mod test {
 
             // First, an one-pass-signature packet.
             if let Some(Packet::OnePassSig(ops)) = message.path_ref(&[0]) {
-                assert_eq!(ops.hash_algo(), HashAlgorithm::SHA256);
+                assert_eq!(ops.hash_algo(), hash);
             } else {
                 panic!("expected an OPS packet");
             }
@@ -2319,7 +2322,7 @@ mod test {
 
             // And, the signature.
             if let Some(Packet::Signature(sig)) = message.path_ref(&[2]) {
-                assert_eq!(sig.hash_algo(), HashAlgorithm::SHA256);
+                assert_eq!(sig.hash_algo(), hash);
             } else {
                 panic!("expected an signature packet");
             }
@@ -2339,7 +2342,7 @@ mod test {
 
             // The signature.
             if let Some(Packet::Signature(sig)) = pp.path_ref(&[0]) {
-                assert_eq!(sig.hash_algo(), HashAlgorithm::SHA256);
+                assert_eq!(sig.hash_algo(), hash);
             } else {
                 panic!("expected an signature packet");
             }
@@ -2347,7 +2350,7 @@ mod test {
         }
 
         f(crate::tests::message("a-problematic-poem.txt.cleartext.sig"),
-          crate::tests::message("a-problematic-poem.txt"))?;
+          crate::tests::message("a-problematic-poem.txt"), HashAlgorithm::SHA256)?;
         f(crate::tests::message("a-cypherpunks-manifesto.txt.cleartext.sig"),
           {
               // The transformation process trims trailing whitespace,
@@ -2358,7 +2361,7 @@ mod test {
               let ws = manifesto.remove(ws_at);
               assert_eq!(ws, b' ');
               manifesto
-          })?;
+          }, HashAlgorithm::SHA256)?;
         Ok(())
     }
 }
