@@ -44,17 +44,13 @@ use crate::{
 ///
 /// A `KeyAmalgamationIter` is returned by [`Cert::keys`].
 ///
-/// [`KeyAmalgamation`]: struct.KeyAmalgamation.html
-/// [`ComponentAmalgamationIter`]: ../struct.ComponentAmalgamationIter.html
-/// [module documentation]: index.html
-/// [`KeyAmalgamationIter::with_policy`]: ../trait.ValidateAmalgamation.html
-/// [`KeyAmalgamation`]: struct.KeyAmalgamation.html
-/// [`ValidKeyAmalgamationIter`]: struct.ValidKeyAmalgamationIter.html
-/// [`ValidKeyAmalgamation`]: struct.ValidKeyAmalgamation.html
-/// [`KeyAmalgamationIter::secret`]: #method.secret
-/// [`KeyAmalgamationIter::unencrypted_secret`]: #method.unencrypted_secret
-/// [`Iterator::filter`]: https://doc.rust-lang.org/stable/std/iter/trait.Iterator.html#method.filter
-/// [`Cert::keys`]: ../../struct.Cert.html#method.keys
+/// [`ComponentAmalgamationIter`]: super::super::ComponentAmalgamationIter
+/// [module documentation]: super
+/// [`KeyAmalgamationIter::with_policy`]: super::ValidateAmalgamation
+/// [`KeyAmalgamationIter::secret`]: KeyAmalgamationIter::secret()
+/// [`KeyAmalgamationIter::unencrypted_secret`]: KeyAmalgamationIter::unencrypted_secret()
+/// [`Iterator::filter`]: std::iter::Iterator::filter()
+/// [`Cert::keys`]: super::super::Cert::keys()
 pub struct KeyAmalgamationIter<'a, P, R>
     where P: key::KeyParts,
           R: key::KeyRole,
@@ -147,10 +143,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
         tracer!(false, "KeyAmalgamationIter::next", 0);
         t!("KeyAmalgamationIter: {:?}", self);
 
-        if self.cert.is_none() {
-            return None;
-        }
-        let cert = self.cert.unwrap();
+        let cert = self.cert?;
 
         loop {
             let ka : ErasedKeyAmalgamation<key::PublicParts>
@@ -182,11 +175,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
                         t!("PK algo is supported... skipping.");
                         continue;
                     }
-                } else {
-                    if want_supported {
-                        t!("PK algo is not supported... skipping.");
-                        continue;
-                    }
+                } else if want_supported {
+                    t!("PK algo is not supported... skipping.");
+                    continue;
                 }
             }
 
@@ -197,11 +188,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
                         t!("Have a secret... skipping.");
                         continue;
                     }
-                } else {
-                    if want_secret {
-                        t!("No secret... skipping.");
-                        continue;
-                    }
+                } else if want_secret {
+                    t!("No secret... skipping.");
+                    continue;
                 }
             }
 
@@ -212,11 +201,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
                             t!("Unencrypted secret... skipping.");
                             continue;
                         }
-                    } else {
-                        if want_unencrypted_secret {
-                            t!("Encrypted secret... skipping.");
-                            continue;
-                        }
+                    } else if want_unencrypted_secret {
+                        t!("Encrypted secret... skipping.");
+                        continue;
                     }
                 } else {
                     // No secret.
@@ -355,9 +342,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`KeyHandle`s]: ../../../enum.KeyHandle.html
-    /// [`key_handles`]: #method.key_handles
-    /// [`KeyHandle::aliases`]: ../../../enum.KeyHandle.html#method.aliases
+    /// [`KeyHandle`s]: super::super::super::KeyHandle
+    /// [`key_handles`]: KeyAmalgamationIter::key_handles()
+    /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
@@ -399,9 +386,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`KeyHandle`s]: ../../../enum.KeyHandle.html
-    /// [`key_handle`]: #method.key_handle
-    /// [`KeyHandle::aliases`]: ../../../enum.KeyHandle.html#method.aliases
+    /// [`KeyHandle`s]: super::super::super::KeyHandle
+    /// [`key_handle`]: KeyAmalgamationIter::key_handle()
+    /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
     pub fn key_handles<'b>(mut self, h: impl Iterator<Item=&'b KeyHandle>)
         -> Self
         where 'a: 'b
@@ -474,8 +461,6 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`ErasedKeyAmalgamation`]: type.ErasedKeyAmalgamation.html
-    /// [`SubordinateKeyAmalgamation`]: type.SubordinateKeyAmalgamation.html
     pub fn subkeys(self) -> KeyAmalgamationIter<'a, P, key::SubordinateRole> {
         KeyAmalgamationIter {
             cert: self.cert,
@@ -526,9 +511,9 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`ValidateAmalgamation`]: ../trait.ValidateAmalgamation.html
-    /// [`alive`]: struct.ValidKeyAmalgamationIter.html#method.alive
-    /// [`revoked`]: struct.ValidKeyAmalgamationIter.html#method.revoked
+    /// [`ValidateAmalgamation`]: super::ValidateAmalgamation
+    /// [`alive`]: ValidKeyAmalgamationIter::alive()
+    /// [`revoked`]: ValidKeyAmalgamationIter::revoked()
     pub fn with_policy<T>(self, policy: &'a dyn Policy, time: T)
         -> ValidKeyAmalgamationIter<'a, P, R>
         where T: Into<Option<SystemTime>>
@@ -539,7 +524,7 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
             subkey_iter: self.subkey_iter,
 
             policy,
-            time: time.into().unwrap_or_else(SystemTime::now),
+            time: time.into().unwrap_or_else(crate::now),
 
             // The filters.
             secret: self.secret,
@@ -628,14 +613,12 @@ impl<'a, P, R> KeyAmalgamationIter<'a, P, R>
 /// # }
 /// ```
 ///
-/// [`ValidKeyAmalgamation`]: struct.ValidKeyAmalgamation.html
-/// [`KeyAmalgamationIter`]: struct.KeyAmalgamationIter.html
-/// [`Policy`]: ../../../policy/trait.Policy.html
-/// [`ValidateAmalgamation`]: ../trait.ValidateAmalgamation.html
-/// [`ValidKeyAmalgamationIter::revoked`]: #method.revoked
-/// [`ValidKeyAmalgamationIter::alive`]: #method.alive
-/// [`KeyAmalgamationIter::with_policy`]: struct.KeyAmalgamationIter.html#method.with_policy
-/// [`ValidCert::keys`]: ../../struct.ValidCert.html#method.keys
+/// [`Policy`]: crate::policy::Policy
+/// [`ValidateAmalgamation`]: super::ValidateAmalgamation
+/// [`ValidKeyAmalgamationIter::revoked`]: ValidKeyAmalgamationIter::revoked()
+/// [`ValidKeyAmalgamationIter::alive`]: ValidKeyAmalgamationIter::alive()
+/// [`KeyAmalgamationIter::with_policy`]: KeyAmalgamationIter::with_policy()
+/// [`ValidCert::keys`]: super::super::ValidCert::keys()
 pub struct ValidKeyAmalgamationIter<'a, P, R>
     where P: key::KeyParts,
           R: key::KeyRole,
@@ -750,10 +733,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
         tracer!(false, "ValidKeyAmalgamationIter::next", 0);
         t!("ValidKeyAmalgamationIter: {:?}", self);
 
-        if self.cert.is_none() {
-            return None;
-        }
-        let cert = self.cert.unwrap();
+        let cert = self.cert?;
 
         if let Some(flags) = self.flags.as_ref() {
             if flags.is_empty() {
@@ -811,11 +791,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
                         t!("PK algo is supported... skipping.");
                         continue;
                     }
-                } else {
-                    if want_supported {
-                        t!("PK algo is not supported... skipping.");
-                        continue;
-                    }
+                } else if want_supported {
+                    t!("PK algo is not supported... skipping.");
+                    continue;
                 }
             }
 
@@ -857,11 +835,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
                         t!("Have a secret... skipping.");
                         continue;
                     }
-                } else {
-                    if want_secret {
-                        t!("No secret... skipping.");
-                        continue;
-                    }
+                } else if want_secret {
+                    t!("No secret... skipping.");
+                    continue;
                 }
             }
 
@@ -872,11 +848,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
                             t!("Unencrypted secret... skipping.");
                             continue;
                         }
-                    } else {
-                        if want_unencrypted_secret {
-                            t!("Encrypted secret... skipping.");
-                            continue;
-                        }
+                    } else if want_unencrypted_secret {
+                        t!("Encrypted secret... skipping.");
+                        continue;
                     }
                 } else {
                     // No secret.
@@ -953,7 +927,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// ```
     ///
     ///   [Section 12.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.2.3.21
-    ///   [`ValidKeyAmalgamation::key_flags`]: struct.ValidKeyAmalgamation.html#method.key_flags
+    ///   [`ValidKeyAmalgamation::key_flags`]: ValidKeyAmalgamation::key_flags()
     pub fn key_flags<F>(mut self, flags: F) -> Self
         where F: Borrow<KeyFlags>
     {
@@ -1020,9 +994,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [`ValidKeyAmalgamation::for_certification`]: struct.ValidKeyAmalgamation.html#method.for_certification
+    ///   [`ValidKeyAmalgamation::for_certification`]: ValidKeyAmalgamation::for_certification()
     ///   [Section 12.1 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-5.2.3.21
-    ///   [`ValidKeyAmalgamation::key_flags`]: struct.ValidKeyAmalgamation.html#method.key_flags
+    ///   [`ValidKeyAmalgamation::key_flags`]: ValidKeyAmalgamation::key_flags()
     pub fn for_certification(self) -> Self {
         self.key_flags(KeyFlags::empty().set_certification())
     }
@@ -1065,7 +1039,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [`ValidKeyAmalgamation::for_signing`]: struct.ValidKeyAmalgamation.html#method.for_signing
+    ///   [`ValidKeyAmalgamation::for_signing`]: ValidKeyAmalgamation::for_signing()
     pub fn for_signing(self) -> Self {
         self.key_flags(KeyFlags::empty().set_signing())
     }
@@ -1108,7 +1082,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [`ValidKeyAmalgamation::for_authentication`]: struct.ValidKeyAmalgamation.html#method.for_authentication
+    ///   [`ValidKeyAmalgamation::for_authentication`]: ValidKeyAmalgamation::for_authentication()
     pub fn for_authentication(self) -> Self {
         self.key_flags(KeyFlags::empty().set_authentication())
     }
@@ -1151,7 +1125,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [`ValidKeyAmalgamation::for_storage_encryption`]: struct.ValidKeyAmalgamation.html#method.for_storage_encryption
+    ///   [`ValidKeyAmalgamation::for_storage_encryption`]: ValidKeyAmalgamation::for_storage_encryption()
     pub fn for_storage_encryption(self) -> Self {
         self.key_flags(KeyFlags::empty().set_storage_encryption())
     }
@@ -1194,7 +1168,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    ///   [`ValidKeyAmalgamation::for_transport_encryption`]: struct.ValidKeyAmalgamation.html#method.for_transport_encryption
+    ///   [`ValidKeyAmalgamation::for_transport_encryption`]: ValidKeyAmalgamation::for_transport_encryption()
     pub fn for_transport_encryption(self) -> Self {
         self.key_flags(KeyFlags::empty().set_transport_encryption())
     }
@@ -1234,7 +1208,7 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    /// [`key_alive`]: ../../../packet/signature/subpacket/struct.SubpacketAreas.html#method.key_alive
+    /// [`key_alive`]: crate::packet::signature::subpacket::SubpacketAreas::key_alive()
     pub fn alive(mut self) -> Self
     {
         self.alive = Some(());
@@ -1466,9 +1440,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`KeyHandle`s]: ../../../enum.KeyHandle.html
-    /// [`key_handles`]: #method.key_handles
-    /// [`KeyHandle::aliases`]: ../../../enum.KeyHandle.html#method.aliases
+    /// [`KeyHandle`s]: super::super::super::KeyHandle
+    /// [`key_handles`]: ValidKeyAmalgamationIter::key_handles()
+    /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
     pub fn key_handle<H>(mut self, h: H) -> Self
         where H: Into<KeyHandle>
     {
@@ -1514,9 +1488,9 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # }
     /// ```
     ///
-    /// [`KeyHandle`s]: ../../../enum.KeyHandle.html
-    /// [`key_handle`]: #method.key_handle
-    /// [`KeyHandle::aliases`]: ../../../enum.KeyHandle.html#method.aliases
+    /// [`KeyHandle`s]: super::super::super::KeyHandle
+    /// [`key_handle`]: ValidKeyAmalgamationIter::key_handle()
+    /// [`KeyHandle::aliases`]: super::super::super::KeyHandle::aliases()
     pub fn key_handles<'b>(mut self, h: impl Iterator<Item=&'b KeyHandle>)
         -> Self
         where 'a: 'b
@@ -1594,8 +1568,6 @@ impl<'a, P, R> ValidKeyAmalgamationIter<'a, P, R>
     /// # Ok(()) }
     /// ```
     ///
-    /// [`ValidErasedKeyAmalgamation`]: type.ValidErasedKeyAmalgamation.html
-    /// [`ValidSubordinateKeyAmalgamation`]: type.ValidSubordinateKeyAmalgamation.html
     pub fn subkeys(self) -> ValidKeyAmalgamationIter<'a, P, key::SubordinateRole> {
         ValidKeyAmalgamationIter {
             cert: self.cert,
@@ -1677,7 +1649,7 @@ mod test {
             .generate().unwrap();
         let flags = KeyFlags::empty().set_transport_encryption();
 
-        let now = SystemTime::now()
+        let now = crate::now()
             - std::time::Duration::new(52 * 7 * 24 * 60 * 60, 0);
         assert_eq!(cert.keys().with_policy(p, now).key_flags(flags).alive().count(),
                    0);
