@@ -163,11 +163,13 @@ impl KeyringValidator {
                 //   Such a packet MUST be ignored when received.
                 return;
             },
+            Tag::Unknown(_) => Token::Unknown(tag, None),
+            Tag::Private(_) => Token::Unknown(tag, None),
             _ => {
                 // Unknown token.
                 self.error = Some(CertParserError::OpenPGP(
                     Error::MalformedMessage(
-                        format!("Invalid Cert: {:?} packet (at {}) not expected",
+                        format!("Invalid Cert: {:?} packet (#{}) not expected",
                                 tag, self.n_packets))));
                 self.tokens.clear();
                 return;
@@ -340,7 +342,14 @@ impl CertValidator {
 /// this way, it is possible to propagate parse errors.
 ///
 /// A `CertParser` returns each [`TPK`] or [`TSK`] that it encounters.
-/// Its behavior can be modeled using a simple state machine.
+/// Note: if you don't actually need all of the certificates, it is
+/// usually faster to use a [`RawCertParser`] and only fully parse and
+/// canonicalize those certificates that are relevant.
+///
+/// [`RawCertParser`]: crate::cert::raw::RawCertParser
+///
+/// A `CertParser`'s behavior can be modeled using a simple state
+/// machine.
 ///
 /// In the first and initial state, it looks for the start of a
 /// certificate, a [`Public Key`] packet or a [`Secret Key`] packet.
@@ -491,7 +500,6 @@ pub struct CertParser<'a> {
     source: Option<Box<dyn Iterator<Item=Result<Packet>> + 'a + Send + Sync>>,
     packets: Vec<Packet>,
     queued_error: Option<anyhow::Error>,
-    saw_error: bool,
     filter: Vec<Box<dyn Send + Sync + Fn(&Cert, bool) -> bool + 'a>>,
 }
 assert_send_and_sync!(CertParser<'_>);
@@ -510,6 +518,7 @@ impl<'a> From<PacketParserResult<'a>> for CertParser<'a>
             let mut retry_with_reader = Box::new(None);
             parser.source = Some(
                 Box::new(std::iter::from_fn(move || {
+                    tracer!(TRACE, "PacketParserResult::next", 0);
                     if let Some(reader) = retry_with_reader.take() {
                         // Try to find the next (armored) blob.
                         match PacketParser::from_buffered_reader(reader) {
@@ -552,9 +561,13 @@ impl<'a> From<PacketParserResult<'a>> for CertParser<'a>
                                         *retry_with_reader =
                                             Some(eof.into_reader()),
                                 }
+                                t!("PacketParser::next yielded a {}",
+                                   packet.tag());
                                 Some(Ok(packet))
                             },
                             Err(err) => {
+                                t!("PacketParser::next returned an error: {}.",
+                                   err);
                                 Some(Err(err))
                             }
                         }
@@ -802,7 +815,6 @@ impl<'a> CertParser<'a> {
         t!("Finalizing certificate with {} packets", n_packets);
 
         // Convert to tokens, but preserve packets if it fails.
-        use std::convert::TryInto;
         let mut failed = false;
         let mut packets: Vec<Packet> = Vec::with_capacity(0);
         let mut tokens: Vec<Token> = Vec::with_capacity(n_packets);
@@ -946,15 +958,16 @@ impl<'a> Iterator for CertParser<'a> {
 
     fn next(&mut self) -> Option<Self::Item> {
         tracer!(TRACE, "CertParser::next", 0);
+        if let Some(err) = self.queued_error.take() {
+            t!("Returning queued error: {}", err);
+            return Some(Err(err));
+        }
 
         loop {
             match self.source.take() {
                 None => {
                     t!("EOF.");
 
-                    if let Some(err) = self.queued_error.take() {
-                        return Some(Err(err));
-                    }
                     if self.packets.is_empty() {
                         return None;
                     }
@@ -992,8 +1005,9 @@ impl<'a> Iterator for CertParser<'a> {
                             self.parse(packet)
                         }
                         Some(Err(err)) => {
+                            self.source = Some(iter);
+
                             t!("Error getting packet: {}", err);
-                            self.saw_error = true;
 
                             if ! self.packets.is_empty() {
                                 // Returned any queued certificate first.
@@ -1005,7 +1019,10 @@ impl<'a> Iterator for CertParser<'a> {
                                     Ok(None) => {
                                         return Some(Err(err));
                                     }
-                                    Err(err) => {
+                                    Err(err2) => {
+                                        // Return the first error,
+                                        // queue the second error.
+                                        self.queued_error = Some(err2);
                                         return Some(Err(err));
                                     }
                                 }
@@ -1055,7 +1072,7 @@ mod test {
     use crate::tests;
 
     #[test]
-    fn tokens() {
+    fn push_tokens() {
         use crate::cert::parser::low_level::lexer::{Token, Lexer};
         use crate::cert::parser::low_level::lexer::Token::*;
         use crate::cert::parser::low_level::CertParser;
@@ -1174,6 +1191,15 @@ mod test {
                 ],
                 result: false,
             },
+            TestVector {
+                s: &[ SecretKey(None), Signature(None),
+                      UserID(None), Signature(None),
+                      SecretSubkey(None), Signature(None),
+                      SecretSubkey(None), Signature(None),
+                      Unknown(Tag::Private(61), None),
+                ],
+                result: true,
+            },
         ];
 
         for v in &test_vectors {
@@ -1195,6 +1221,68 @@ mod test {
         }
     }
 
+    #[test]
+    fn push_tags() {
+        use Tag::*;
+
+        struct TestVector<'a> {
+            s: &'a [Tag],
+            result: bool,
+        }
+
+        let test_vectors = [
+            TestVector {
+                s: &[ PublicKey ],
+                result: true,
+            },
+            TestVector {
+                s: &[ SecretKey, Signature,
+                      UserID, Signature,
+                      SecretSubkey, Signature,
+                      SecretSubkey, Signature,
+                      Tag::Private(61),
+                ],
+                result: true,
+            },
+            TestVector {
+                s: &[ SecretKey, Signature,
+                      UserID, Signature,
+                      SecretSubkey, Signature,
+                      SecretSubkey, Signature,
+                      Tag::Unknown(61),
+                ],
+                result: true,
+            },
+            TestVector {
+                s: &[ SecretKey, Signature,
+                      UserID, Signature,
+                      SecretSubkey, Signature,
+                      SecretSubkey, Signature,
+                      Tag::Unknown(61),
+                      SecretKey, Signature,
+                      UserID, Signature,
+                      SecretSubkey, Signature,
+                      SecretSubkey, Signature,
+                ],
+                // This is a keyring, not a cert.
+                result: false,
+            },
+        ];
+
+        for v in &test_vectors {
+            if v.result {
+                let mut l = CertValidator::new();
+                for &tag in v.s.into_iter() {
+                    l.push(tag.clone());
+                    assert_match!(CertValidity::CertPrefix = l.check());
+                }
+
+                l.finish();
+                assert_match!(CertValidity::Cert = l.check());
+            }
+        }
+    }
+
     #[test]
     fn marker_packet_ignored() {
         use crate::serialize::Serialize;
@@ -1631,16 +1719,8 @@ mod test {
         {
             let ps_orig: Vec<Packet> = c_orig.into_packets().collect();
             let ps_parsed: Vec<Packet> = c_parsed.into_packets().collect();
-            if bad > 0 && ! literal && i == n - 1 {
-                // On a parse error, we lose the last successfully
-                // parsed packet.  This is annoying.  But, the
-                // file is corrupted anyway, so...
-                assert_eq!(ps_orig.len() - 1, ps_parsed.len(),
-                           "number of packets: expected vs. got");
-            } else {
-                assert_eq!(ps_orig.len(), ps_parsed.len(),
-                           "number of packets: expected vs. got");
-            }
+            assert_eq!(ps_orig.len(), ps_parsed.len(),
+                       "number of packets: expected vs. got");
 
             for (j, (p_orig, p_parsed)) in
                 ps_orig
@@ -1778,4 +1858,36 @@ mod test {
         assert_eq!(certs.len(), 1);
         assert!(certs[0].fingerprint() == fp);
     }
+
+    #[test]
+    fn packet_source_includes_an_error() -> Result<()> {
+        let mut ppr
+            = PacketParser::from_bytes(crate::tests::key("testy.pgp"))?;
+        let mut testy = Vec::new();
+        while let PacketParserResult::Some(pp) = ppr {
+            let (packet, ppr_) = pp.next()?;
+            testy.push(packet);
+            ppr = ppr_;
+        }
+
+        // A cert, two errors, another cert.
+        let mut packets: Vec<Result<Packet>> = Vec::new();
+        for p in testy.iter() {
+            packets.push(Ok(p.clone()));
+        }
+        packets.push(Err(anyhow::anyhow!("An error")));
+        packets.push(Err(anyhow::anyhow!("Another error")));
+        for p in testy.iter() {
+            packets.push(Ok(p.clone()));
+        }
+
+        let certs = CertParser::from(packets).collect::<Vec<Result<Cert>>>();
+        assert_eq!(certs.len(), 4);
+        assert!(certs[0].is_ok());
+        assert!(certs[1].is_err());
+        assert!(certs[2].is_err());
+        assert!(certs[3].is_ok());
+
+        Ok(())
+    }
 }
