@@ -15,7 +15,6 @@ use crate::utils::{
 use crate::Error;
 use crate::Result;
 use crate::crypto::SessionKey;
-use crate::crypto::mem::secure_cmp;
 use crate::seal;
 use crate::parse::Cookie;
 
@@ -32,12 +31,6 @@ const MAX_CHUNK_SIZE: usize = 1 << 22; // 4MiB
 /// Maximum size of any Nonce used by an AEAD mode.
 pub const MAX_NONCE_LEN: usize = 16;
 
-/// Disables authentication checks.
-///
-/// This is DANGEROUS, and is only useful for debugging problems with
-/// malformed AEAD-encrypted messages.
-const DANGER_DISABLE_AUTHENTICATION: bool = false;
-
 /// Converts a chunk size to a usize.
 pub(crate) fn chunk_size_usize(chunk_size: u64) -> Result<usize> {
     chunk_size.try_into()
@@ -57,19 +50,18 @@ pub(crate) fn chunk_size_usize(chunk_size: u64) -> Result<usize> {
 ///
 /// [sealed]: https://rust-lang.github.io/api-guidelines/future-proofing.html#sealed-traits-protect-against-downstream-implementations-c-sealed
 pub trait Aead : seal::Sealed {
-    /// Adds associated data `ad`.
-    fn update(&mut self, ad: &[u8]);
-
-    /// Encrypts one block `src` to `dst`.
-    fn encrypt(&mut self, dst: &mut [u8], src: &[u8]);
-    /// Decrypts one block `src` to `dst`.
-    fn decrypt(&mut self, dst: &mut [u8], src: &[u8]);
-
-    /// Produce the digest.
-    fn digest(&mut self, digest: &mut [u8]);
+    /// Encrypts one chunk `src` to `dst` adding a digest.
+    ///
+    /// Note: `dst` must be large enough to accommodate both the
+    /// ciphertext and the digest!
+    fn encrypt_seal(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()>;
 
     /// Length of the digest in bytes.
     fn digest_size(&self) -> usize;
+
+    /// Decrypt one chunk `src` to `dst` and verify that the digest is
+    /// correct.
+    fn decrypt_verify(&mut self, dst: &mut [u8], src: &[u8]) -> Result<()>;
 }
 
 /// Whether AEAD cipher is used for data encryption or decryption.
@@ -299,13 +291,8 @@ impl<'a, S: Schedule> Decryptor<'a, S> {
     // is less than `plaintext.len()`, then it is either because we
     // reached the end of the input or an error occurred.
     fn read_helper(&mut self, plaintext: &mut [u8]) -> Result<usize> {
-        use std::cmp::Ordering;
-
         let mut pos = 0;
 
-        // Buffer to hold a digest.
-        let mut digest = vec![0u8; self.digest_size];
-
         // 1. Copy any buffered data.
         if !self.buffer.is_empty() {
             let to_copy = cmp::min(self.buffer.len(), plaintext.len());
@@ -397,12 +384,8 @@ impl<'a, S: Schedule> Decryptor<'a, S> {
                 let mut aead = self.schedule.next_chunk(
                     self.chunk_index,
                     |iv, ad| {
-                        self.aead.context(self.sym_algo, &self.key, iv,
+                        self.aead.context(self.sym_algo, &self.key, ad, iv,
                                           CipherOp::Decrypt)
-                            .map(|mut aead| {
-                                aead.update(ad);
-                                aead
-                            })
                     })?;
 
                 // Decrypt the chunk and check the tag.
@@ -418,15 +401,7 @@ impl<'a, S: Schedule> Decryptor<'a, S> {
                     &mut plaintext[pos..pos + to_decrypt]
                 };
 
-                aead.decrypt(buffer, &chunk[..to_decrypt]);
-
-                // Check digest.
-                aead.digest(&mut digest);
-                if secure_cmp(&digest[..], &chunk[to_decrypt..])
-                    != Ordering::Equal && ! DANGER_DISABLE_AUTHENTICATION
-                {
-                    return Err(Error::ManipulatedMessage.into());
-                }
+                aead.decrypt_verify(buffer, chunk)?;
 
                 if double_buffer {
                     let to_copy = plaintext.len() - pos;
@@ -456,23 +431,13 @@ impl<'a, S: Schedule> Decryptor<'a, S> {
                 let mut aead = self.schedule.final_chunk(
                     self.chunk_index, self.bytes_decrypted,
                     |iv, ad| {
-                        self.aead.context(self.sym_algo, &self.key, iv,
+                        self.aead.context(self.sym_algo, &self.key, ad, iv,
                                           CipherOp::Decrypt)
-                            .map(|mut aead| {
-                                aead.update(ad);
-                                aead
-                            })
                     })?;
 
-                aead.digest(&mut digest);
-
                 let final_digest = self.source.data(final_digest_size)?;
-                if final_digest.len() != final_digest_size
-                    || secure_cmp(&digest[..], final_digest) != Ordering::Equal
-                    && ! DANGER_DISABLE_AUTHENTICATION
-                {
-                    return Err(Error::ManipulatedMessage.into());
-                }
+
+                aead.decrypt_verify(&mut [], final_digest)?;
 
                 // Consume the data only on success so that we keep
                 // returning the error.
@@ -657,7 +622,7 @@ impl<W: io::Write, S: Schedule> Encryptor<W, S> {
             chunk_index: 0,
             bytes_encrypted: 0,
             buffer: Vec::with_capacity(chunk_size),
-            scratch: vec![0; chunk_size],
+            scratch: vec![0; chunk_size + aead.digest_size()?],
         })
     }
 
@@ -680,26 +645,19 @@ impl<W: io::Write, S: Schedule> Encryptor<W, S> {
             if self.buffer.len() == self.chunk_size {
                 let mut aead =
                     self.schedule.next_chunk(self.chunk_index, |iv, ad| {
-                        self.aead.context(self.sym_algo, &self.key, iv,
+                        self.aead.context(self.sym_algo, &self.key, ad, iv,
                                           CipherOp::Encrypt)
-                            .map(|mut aead| {
-                                aead.update(ad);
-                                aead
-                            })
                     })?;
 
                 let inner = self.inner.as_mut().unwrap();
 
                 // Encrypt the chunk.
-                aead.encrypt(&mut self.scratch, &self.buffer);
-                self.bytes_encrypted += self.scratch.len() as u64;
+                aead.encrypt_seal(&mut self.scratch, &self.buffer)?;
+                self.bytes_encrypted += self.chunk_size as u64;
                 self.chunk_index += 1;
+                // XXX: clear plaintext buffer.
                 crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
-
-                // Write digest.
-                aead.digest(&mut self.scratch[..self.digest_size]);
-                inner.write_all(&self.scratch[..self.digest_size])?;
             }
         }
 
@@ -709,25 +667,17 @@ impl<W: io::Write, S: Schedule> Encryptor<W, S> {
                 // Complete chunk.
                 let mut aead =
                     self.schedule.next_chunk(self.chunk_index, |iv, ad| {
-                        self.aead.context(self.sym_algo, &self.key, iv,
+                        self.aead.context(self.sym_algo, &self.key, ad, iv,
                                           CipherOp::Encrypt)
-                            .map(|mut aead| {
-                                aead.update(ad);
-                                aead
-                            })
                     })?;
 
                 let inner = self.inner.as_mut().unwrap();
 
                 // Encrypt the chunk.
-                aead.encrypt(&mut self.scratch, chunk);
-                self.bytes_encrypted += self.scratch.len() as u64;
+                aead.encrypt_seal(&mut self.scratch, chunk)?;
+                self.bytes_encrypted += self.chunk_size as u64;
                 self.chunk_index += 1;
                 inner.write_all(&self.scratch)?;
-
-                // Write digest.
-                aead.digest(&mut self.scratch[..self.digest_size]);
-                inner.write_all(&self.scratch[..self.digest_size])?;
             } else {
                 // Stash for later.
                 assert!(self.buffer.is_empty());
@@ -744,40 +694,35 @@ impl<W: io::Write, S: Schedule> Encryptor<W, S> {
             if !self.buffer.is_empty() {
                 let mut aead =
                     self.schedule.next_chunk(self.chunk_index, |iv, ad| {
-                        self.aead.context(self.sym_algo, &self.key, iv,
+                        self.aead.context(self.sym_algo, &self.key, ad, iv,
                                           CipherOp::Encrypt)
-                            .map(|mut aead| {
-                                aead.update(ad);
-                                aead
-                            })
                     })?;
 
                 // Encrypt the chunk.
-                unsafe { self.scratch.set_len(self.buffer.len()) }
-                aead.encrypt(&mut self.scratch, &self.buffer);
-                self.bytes_encrypted += self.scratch.len() as u64;
+                unsafe {
+                    // Safety: remaining data is less than the chunk
+                    // size.  The vector has capacity chunk size plus
+                    // digest size.
+                    debug_assert!(self.buffer.len() < self.chunk_size);
+                    self.scratch.set_len(self.buffer.len() + self.digest_size)
+                }
+                aead.encrypt_seal(&mut self.scratch, &self.buffer)?;
+                self.bytes_encrypted += self.buffer.len() as u64;
                 self.chunk_index += 1;
+                // XXX: clear plaintext buffer
                 crate::vec_truncate(&mut self.buffer, 0);
                 inner.write_all(&self.scratch)?;
-
-                // Write digest.
-                unsafe { self.scratch.set_len(self.digest_size) }
-                aead.digest(&mut self.scratch[..self.digest_size]);
-                inner.write_all(&self.scratch[..self.digest_size])?;
             }
 
             // Write final digest.
             let mut aead = self.schedule.final_chunk(
                 self.chunk_index, self.bytes_encrypted,
                 |iv, ad| {
-                    self.aead.context(self.sym_algo, &self.key, iv,
+                    self.aead.context(self.sym_algo, &self.key, ad, iv,
                                       CipherOp::Encrypt)
-                        .map(|mut aead| {
-                            aead.update(ad);
-                            aead
-                        })
                 })?;
-            aead.digest(&mut self.scratch[..self.digest_size]);
+            debug_assert!(self.digest_size <= self.scratch.len());
+            aead.encrypt_seal(&mut self.scratch[..self.digest_size], b"")?;
             inner.write_all(&self.scratch[..self.digest_size])?;
 
             Ok(inner)
@@ -856,17 +801,10 @@ mod tests {
                          .iter()
                          .filter(|algo| algo.is_supported()) {
 
-            if cfg!(feature = "crypto-rust")
-                && sym_algo == &SymmetricAlgorithm::Twofish {
-                    eprintln!("XXX: Skipping Twofish until Twofish \
-                               implements Clone");
-                    continue;
-            }
-
             for aead in [
                 AEADAlgorithm::EAX,
                 AEADAlgorithm::OCB,
-            ].iter().filter(|algo| algo.is_supported()) {
+            ].iter().filter(|algo| algo.is_supported() && algo.supports_symmetric_algo(sym_algo)) {
                 let chunk_size = 64;
                 let mut key = vec![0; sym_algo.key_size().unwrap()];
                 crate::crypto::random(&mut key);
