@@ -13,7 +13,7 @@ use crate::crypto::SessionKey;
 use crate::crypto::{pad, pad_at_least, pad_truncating};
 use crate::packet::key::{Key4, SecretParts};
 use crate::packet::{key, Key};
-use crate::types::{PublicKeyAlgorithm, SymmetricAlgorithm};
+use crate::types::PublicKeyAlgorithm;
 use crate::types::{Curve, HashAlgorithm};
 
 use num_bigint_dig::{traits::ModInverse, BigInt, BigUint};
@@ -234,6 +234,9 @@ impl Signer for KeyPair {
                             // so we can't use an empty buffer here.
                             let (count, seed) = ([0x0; 4], vec![0x0; q.value().len()]);
 
+                            let group_size = std::cmp::min(q.value().len(), 32);
+                            let key_size = y.len();
+
                             DsaPrivateBlob::V2(Blob::<DsaKeyPrivateV2Blob>::clone_from_parts(
                                 &winapi::shared::bcrypt::BCRYPT_DSA_KEY_BLOB_V2 {
                                     dwMagic: winapi::shared::bcrypt::BCRYPT_DSA_PRIVATE_MAGIC_V2,
@@ -242,8 +245,8 @@ impl Signer for KeyPair {
                                     // Currently, if the key is less than 128
                                     // bits, q is 20 bytes long.
                                     // If the key exceeds 256 bits, q is 32 bytes long.
-                                    cbGroupSize: std::cmp::min(q.value().len(), 32) as u32,
-                                    cbKey: y.len() as u32,
+                                    cbGroupSize: group_size as u32,
+                                    cbKey: key_size as u32,
                                     cbSeedLength: seed.len() as u32,
                                     hashAlgorithm: hash,
                                     standardVersion: 1, // FIPS 186-3
@@ -251,11 +254,11 @@ impl Signer for KeyPair {
                                 },
                                 &DsaKeyPrivateV2Payload {
                                     seed: &seed,
-                                    group: q.value(),
-                                    modulus: p.value(),
-                                    generator: g.value(),
+                                    group: &q.value_padded(group_size)?,
+                                    modulus: &p.value_padded(key_size)?,
+                                    generator: &g.value_padded(key_size)?,
                                     public: &y,
-                                    priv_exp: x.value(),
+                                    priv_exp: &x.value_padded(group_size),
                                 },
                             ))
                         },
@@ -408,8 +411,17 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                     },
                 }
             },
+
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(), data),
-            algo => Err(Error::UnsupportedPublicKeyAlgorithm(algo).into()),
+
+            RSASign | DSA | ECDSA | EdDSA =>
+                Err(Error::InvalidOperation(
+                    format!("{} is not an encryption algorithm", self.pk_algo())
+                ).into()),
+
+            ElGamalEncrypt | ElGamalEncryptSign |
+            Private(_) | Unknown(_) =>
+                Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
         }
     }
 
@@ -681,22 +693,7 @@ impl<R> Key4<SecretParts, R>
 where
     R: key::KeyRole,
 {
-    /// Creates a new OpenPGP secret key packet for an existing X25519 key.
-    ///
-    /// The ECDH key will use hash algorithm `hash` and symmetric
-    /// algorithm `sym`.  If one or both are `None` secure defaults
-    /// will be used.  The key will have its creation date set to
-    /// `ctime` or the current time if `None` is given.
-    pub fn import_secret_cv25519<H, S, T>(
-        private_key: &[u8],
-        hash: H,
-        sym: S,
-        ctime: T,
-    ) -> Result<Self>
-    where
-        H: Into<Option<HashAlgorithm>>,
-        S: Into<Option<SymmetricAlgorithm>>,
-        T: Into<Option<SystemTime>>,
+    pub(crate) fn derive_cv25519_public_key(private_key: &Protected) -> Result<[u8; 32]>
     {
         use cng::asymmetric::{AsymmetricAlgorithm, AsymmetricAlgorithmId, Ecdh, Private};
         use cng::asymmetric::{AsymmetricKey, Export};
@@ -709,33 +706,7 @@ where
             &provider,
             private_key
         )?;
-        let blob = key.export()?;
-
-        // Mark MPI as compressed point with 0x40 prefix. See
-        // https://tools.ietf.org/html/draft-ietf-openpgp-rfc4880bis-07#section-13.2.
-        let mut public = [0u8; 1 + CURVE25519_SIZE];
-        public[0] = 0x40;
-        public[1..].copy_from_slice(blob.x());
-
-        // Reverse the scalar.  See
-        // https://lists.gnupg.org/pipermail/gnupg-devel/2018-February/033437.html.
-        let mut private = blob.d().to_vec();
-        private.reverse();
-
-        use crate::crypto::ecdh;
-        Self::with_secret(
-            ctime.into().unwrap_or_else(crate::now),
-            PublicKeyAlgorithm::ECDH,
-            mpi::PublicKey::ECDH {
-                curve: Curve::Cv25519,
-                hash: hash.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kdf_hash(&Curve::Cv25519)),
-                sym: sym.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kek_cipher(&Curve::Cv25519)),
-                q: mpi::MPI::new(&public),
-            },
-            mpi::SecretKeyMaterial::ECDH { scalar: private.into() }.into()
-        )
+        Ok(<[u8; 32]>::try_from(&key.export()?.x()[..])?)
     }
 
     /// Creates a new OpenPGP secret key packet for an existing Ed25519 key.
@@ -767,7 +738,7 @@ where
                 q: mpi::MPI::new(&public)
             },
             mpi::SecretKeyMaterial::EdDSA {
-                scalar: mpi::MPI::new(&private_key).into(),
+                scalar: private_key.into(),
             }.into()
         )
     }
@@ -807,10 +778,10 @@ where
                 n: mpi::MPI::new(&n.to_bytes_be()),
             },
             mpi::SecretKeyMaterial::RSA {
-                d: mpi::MPI::new(d).into(),
-                p: mpi::MPI::new(p).into(),
-                q: mpi::MPI::new(q).into(),
-                u: mpi::MPI::new(&u.to_bytes_be()).into(),
+                d: d.into(),
+                p: p.into(),
+                q: q.into(),
+                u: u.to_bytes_be().into(),
             }.into()
         )
     }
@@ -844,8 +815,8 @@ where
         let private = mpi::SecretKeyMaterial::RSA {
             p: p.into(),
             q: q.into(),
-            d: mpi::MPI::new(blob.priv_exp()).into(),
-            u: mpi::MPI::new(&u.to_bytes_be()).into(),
+            d: blob.priv_exp().into(),
+            u: u.to_bytes_be().into(),
         };
 
         Self::with_secret(
