@@ -222,9 +222,18 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                     },
                 }
             },
+
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(),
                                                  data),
-            algo => Err(Error::UnsupportedPublicKeyAlgorithm(algo).into()),
+
+            RSASign | DSA | ECDSA | EdDSA =>
+                Err(Error::InvalidOperation(
+                    format!("{} is not an encryption algorithm", self.pk_algo())
+                ).into()),
+
+            ElGamalEncrypt | ElGamalEncryptSign |
+            Private(_) | Unknown(_) =>
+                Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
         }
     }
 
@@ -319,45 +328,16 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 use std::time::SystemTime;
 use crate::crypto::mem::Protected;
 use crate::packet::key::{Key4, SecretParts};
-use crate::types::{PublicKeyAlgorithm, SymmetricAlgorithm};
+use crate::types::PublicKeyAlgorithm;
 
 impl<R> Key4<SecretParts, R>
     where R: key::KeyRole,
 {
-
-    /// Creates a new OpenPGP secret key packet for an existing X25519 key.
-    ///
-    /// The ECDH key will use hash algorithm `hash` and symmetric
-    /// algorithm `sym`.  If one or both are `None` secure defaults
-    /// will be used.  The key will have it's creation date set to
-    /// `ctime` or the current time if `None` is given.
-    pub fn import_secret_cv25519<H, S, T>(private_key: &[u8],
-                                          hash: H, sym: S, ctime: T)
-        -> Result<Self> where H: Into<Option<HashAlgorithm>>,
-                              S: Into<Option<SymmetricAlgorithm>>,
-                              T: Into<Option<SystemTime>>
+    pub(crate) fn derive_cv25519_public_key(private_key: &Protected) -> Result<[u8; 32]>
     {
-        let mut public_key = [0; curve25519::CURVE25519_SIZE];
-        curve25519::mul_g(&mut public_key, private_key).unwrap();
-
-        let mut private_key = Vec::from(private_key);
-        private_key.reverse();
-
-        use crate::crypto::ecdh;
-        Self::with_secret(
-            ctime.into().unwrap_or_else(crate::now),
-            PublicKeyAlgorithm::ECDH,
-            mpi::PublicKey::ECDH {
-                curve: Curve::Cv25519,
-                hash: hash.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kdf_hash(&Curve::Cv25519)),
-                sym: sym.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kek_cipher(&Curve::Cv25519)),
-                q: MPI::new_compressed_point(&public_key),
-            },
-            mpi::SecretKeyMaterial::ECDH {
-                scalar: private_key.into(),
-            }.into())
+        let mut public_key = [0; ed25519::ED25519_KEY_SIZE];
+        curve25519::mul_g(&mut public_key, private_key)?;
+        Ok(public_key)
     }
 
     /// Creates a new OpenPGP secret key packet for an existing Ed25519 key.
@@ -380,7 +360,7 @@ impl<R> Key4<SecretParts, R>
                 q: MPI::new_compressed_point(&public_key),
             },
             mpi::SecretKeyMaterial::EdDSA {
-                scalar: mpi::MPI::new(private_key).into(),
+                scalar: private_key.into(),
             }.into())
     }
 
@@ -405,10 +385,10 @@ impl<R> Key4<SecretParts, R>
                 n: mpi::MPI::new(&key.n()[..]),
             },
             mpi::SecretKeyMaterial::RSA {
-                d: mpi::MPI::new(d).into(),
-                p: mpi::MPI::new(&a[..]).into(),
-                q: mpi::MPI::new(&b[..]).into(),
-                u: mpi::MPI::new(&c[..]).into(),
+                d: d.into(),
+                p: a.into(),
+                q: b.into(),
+                u: c.into(),
             }.into())
     }
 
@@ -423,10 +403,10 @@ impl<R> Key4<SecretParts, R>
             n: MPI::new(&*public.n()),
         };
         let private_mpis = mpi::SecretKeyMaterial::RSA {
-            d: MPI::new(&*private.d()).into(),
-            p: MPI::new(&*p).into(),
-            q: MPI::new(&*q).into(),
-            u: MPI::new(&*u).into(),
+            d: private.d().into(),
+            p: p.into(),
+            q: q.into(),
+            u: u.into(),
         };
 
         Self::with_secret(
@@ -518,7 +498,7 @@ impl<R> Key4<SecretParts, R>
                     q: MPI::new_point(&pub_x, &pub_y, field_sz),
                 };
                 let private_mpis = mpi::SecretKeyMaterial::ECDSA{
-                    scalar: MPI::new(&private.as_bytes()).into(),
+                    scalar: private.as_bytes().into(),
                 };
                 let sec = private_mpis.into();
 
@@ -557,7 +537,7 @@ impl<R> Key4<SecretParts, R>
                         sym,
                     };
                     let private_mpis = mpi::SecretKeyMaterial::ECDH{
-                        scalar: MPI::new(&private.as_bytes()).into(),
+                        scalar: private.as_bytes().into(),
                     };
                     let sec = private_mpis.into();
 
