@@ -7,9 +7,10 @@
 use std::convert::TryFrom;
 use std::time::SystemTime;
 
+use x25519_dalek_ng as x25519_dalek;
 use num_bigint_dig::{traits::ModInverse, BigUint};
-use rand07::rngs::OsRng;
-use rsa::{PaddingScheme, RSAPublicKey, RSAPrivateKey, PublicKey, PublicKeyParts, Hash};
+use rsa::traits::{PrivateKeyParts, PublicKeyParts};
+use rsa::{Pkcs1v15Encrypt, RsaPublicKey, RsaPrivateKey, Pkcs1v15Sign};
 
 use crate::{Error, Result};
 use crate::crypto::asymmetric::{KeyPair, Decryptor, Signer};
@@ -19,43 +20,43 @@ use crate::crypto::SessionKey;
 use crate::crypto::pad_truncating;
 use crate::packet::{key, Key};
 use crate::packet::key::{Key4, SecretParts};
-use crate::types::{Curve, HashAlgorithm, PublicKeyAlgorithm, SymmetricAlgorithm};
+use crate::types::{Curve, HashAlgorithm, PublicKeyAlgorithm};
+
+use super::GenericArrayExt;
 
 const CURVE25519_SIZE: usize = 32;
 
-fn pkcs1_padding(hash_algo: HashAlgorithm) -> Result<PaddingScheme> {
+fn pkcs1_padding(hash_algo: HashAlgorithm) -> Result<Pkcs1v15Sign> {
     let hash = match hash_algo {
-        HashAlgorithm::MD5 => Hash::MD5,
-        HashAlgorithm::SHA1 => Hash::SHA1,
-        HashAlgorithm::SHA224 => Hash::SHA2_224,
-        HashAlgorithm::SHA256 => Hash::SHA2_256,
-        HashAlgorithm::SHA384 => Hash::SHA2_384,
-        HashAlgorithm::SHA512 => Hash::SHA2_512,
-        HashAlgorithm::RipeMD => Hash::RIPEMD160,
+        HashAlgorithm::MD5 => Pkcs1v15Sign::new::<md5::Md5>(),
+        HashAlgorithm::SHA1 => Pkcs1v15Sign::new::<sha1::Sha1>(),
+        HashAlgorithm::SHA224 => Pkcs1v15Sign::new::<sha2::Sha224>(),
+        HashAlgorithm::SHA256 => Pkcs1v15Sign::new::<sha2::Sha256>(),
+        HashAlgorithm::SHA384 => Pkcs1v15Sign::new::<sha2::Sha384>(),
+        HashAlgorithm::SHA512 => Pkcs1v15Sign::new::<sha2::Sha512>(),
+        HashAlgorithm::RipeMD => Pkcs1v15Sign::new::<ripemd::Ripemd160>(),
         _ => return Err(Error::InvalidArgument(format!(
             "Algorithm {:?} not representable", hash_algo)).into()),
     };
-    Ok(PaddingScheme::PKCS1v15Sign {
-        hash: Some(hash)
-    })
+    Ok(hash)
 }
 
-fn rsa_public_key(e: &MPI, n: &MPI) -> Result<RSAPublicKey> {
+fn rsa_public_key(e: &MPI, n: &MPI) -> Result<RsaPublicKey> {
     let n = BigUint::from_bytes_be(n.value());
     let e = BigUint::from_bytes_be(e.value());
-    Ok(RSAPublicKey::new(n, e)?)
+    Ok(RsaPublicKey::new(n, e)?)
 }
 
 #[allow(clippy::many_single_char_names)]
 fn rsa_private_key(e: &MPI, n: &MPI, p: &ProtectedMPI, q: &ProtectedMPI, d: &ProtectedMPI)
-    -> RSAPrivateKey
+    -> Result<RsaPrivateKey>
 {
     let n = BigUint::from_bytes_be(n.value());
     let e = BigUint::from_bytes_be(e.value());
     let p = BigUint::from_bytes_be(p.value());
     let q = BigUint::from_bytes_be(q.value());
     let d = BigUint::from_bytes_be(d.value());
-    RSAPrivateKey::from_components(n, e, d, vec![p, q])
+    Ok(RsaPrivateKey::from_components(n, e, d, vec![p, q])?)
 }
 
 impl Signer for KeyPair {
@@ -75,7 +76,7 @@ impl Signer for KeyPair {
             (RSASign,
              mpi::PublicKey::RSA { e, n },
              mpi::SecretKeyMaterial::RSA { p, q, d, .. }) => {
-                let key = rsa_private_key(e, n, p, q, d);
+                let key = rsa_private_key(e, n, p, q, d)?;
                 let padding = pkcs1_padding(hash_algo)?;
                 let sig = key.sign(padding, digest)?;
                 Ok(mpi::Signature::RSA {
@@ -97,6 +98,7 @@ impl Signer for KeyPair {
                     use p256::{
                         elliptic_curve::{
                             generic_array::GenericArray as GA,
+                            ops::Reduce,
                         },
                         Scalar,
                     };
@@ -105,18 +107,18 @@ impl Signer for KeyPair {
                     };
 
                     const LEN: usize = 32;
-                    let key = Scalar::from_bytes_reduced(
-                        GA::from_slice(&scalar.value_padded(LEN)));
-                    let dig = Scalar::from_bytes_reduced(
-                        GA::from_slice(&pad_truncating(digest, LEN)));
+                    let key = scalar.value_padded(LEN);
+                    let key = Scalar::reduce_bytes(GA::try_from_slice(&key)?);
+                    let dig = pad_truncating(digest, LEN);
+                    let dig = GA::try_from_slice(&dig)?;
 
                     let sig = loop {
                         let mut k: Protected = vec![0; LEN].into();
                         crate::crypto::random(&mut k);
-                        let k = Scalar::from_bytes_reduced(
-                            GA::from_slice(&k));
-                        if let Ok(s) = key.try_sign_prehashed(&k, &dig) {
-                            break s;
+                        let k = Scalar::reduce_bytes(
+                            GA::try_from_slice(&k)?);
+                        if let Ok(s) = key.try_sign_prehashed(k, &dig) {
+                            break s.0;
                         }
                     };
 
@@ -193,9 +195,8 @@ impl Decryptor for KeyPair {
             (mpi::PublicKey::RSA { e, n },
              mpi::SecretKeyMaterial::RSA { p, q, d, .. },
              mpi::Ciphertext::RSA { c }) => {
-                let key = rsa_private_key(e, n, p, q, d);
-                let padding = PaddingScheme::PKCS1v15Encrypt;
-                let decrypted = key.decrypt(padding, c.value())?;
+                let key = rsa_private_key(e, n, p, q, d)?;
+                let decrypted = key.decrypt(Pkcs1v15Encrypt, c.value())?;
                 Ok(SessionKey::from(decrypted))
             }
 
@@ -234,8 +235,9 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                             "Plaintext data too large".into()).into());
                     }
                     let key = rsa_public_key(e, n)?;
-                    let padding = PaddingScheme::PKCS1v15Encrypt;
-                    let ciphertext = key.encrypt(&mut OsRng, padding, data.as_ref())?;
+                    let ciphertext = key.encrypt(
+                        &mut rsa::rand_core::OsRng,
+                        Pkcs1v15Encrypt, data.as_ref())?;
                     Ok(mpi::Ciphertext::RSA {
                         c: mpi::MPI::new(&ciphertext)
                     })
@@ -243,8 +245,17 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                 pk => Err(Error::MalformedPacket(format!(
                     "Key: Expected RSA public key, got {:?}", pk)).into())
             }
+
             ECDH => crate::crypto::ecdh::encrypt(self.parts_as_public(), data),
-            algo => Err(Error::UnsupportedPublicKeyAlgorithm(algo).into()),
+
+            RSASign | DSA | ECDSA | EdDSA =>
+                Err(Error::InvalidOperation(
+                    format!("{} is not an encryption algorithm", self.pk_algo())
+                ).into()),
+
+            ElGamalEncrypt | ElGamalEncryptSign |
+            Private(_) | Unknown(_) =>
+                Err(Error::UnsupportedPublicKeyAlgorithm(self.pk_algo()).into()),
         }
     }
 
@@ -259,7 +270,22 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
             (mpi::PublicKey::RSA { e, n }, mpi::Signature::RSA { s }) => {
                 let key = rsa_public_key(e, n)?;
                 let padding = pkcs1_padding(hash_algo)?;
-                key.verify(padding, digest, s.value())?;
+                // Originally, we had:
+                //
+                // key.verify(padding, digest, s.value())?;
+                //
+                // Since version 0.9.0 of the rsa crate, this no
+                // longer works, because the verify function checks
+                // that the signature length in bytes is the same as
+                // the key length.  No other crypto backend appears
+                // care (including older version of the rsa crate),
+                // but would happily left pad it with zeros.  We now
+                // do that manually:
+                //
+                // See
+                // https://docs.rs/rsa/0.9.0/src/rsa/pkcs1v15.rs.html#212
+                // and https://github.com/RustCrypto/RSA/issues/322.
+                key.verify(padding, digest, &s.value_padded(key.size())?)?;
                 Ok(())
             }
             (mpi::PublicKey::DSA { .. },
@@ -277,7 +303,6 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                             generic_array::GenericArray as GA,
                             sec1::FromEncodedPoint,
                         },
-                        Scalar,
                     };
                     use ecdsa::{
                         EncodedPoint,
@@ -286,17 +311,22 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
                     const LEN: usize = 32;
 
                     let key = AffinePoint::from_encoded_point(
-                        &EncodedPoint::from_bytes(q.value())?)
-                        .ok_or_else(|| Error::InvalidKey(
-                            "Point is not on the curve".into()))?;
+                        &EncodedPoint::<p256::NistP256>::from_bytes(q.value())?);
+                    let key = if key.is_some().into() {
+                        key.unwrap()
+                    } else {
+                        return Err(Error::InvalidKey(
+                            "Point is not on the curve".into()).into());
+                    };
+
                     let sig = Signature::from_scalars(
-                        Scalar::from_bytes_reduced(
-                            GA::from_slice(&r.value_padded(LEN).map_err(bad)?)),
-                        Scalar::from_bytes_reduced(
-                            GA::from_slice(&s.value_padded(LEN).map_err(bad)?)))
+                        GA::try_clone_from_slice(
+                            &r.value_padded(LEN).map_err(bad)?)?,
+                        GA::try_clone_from_slice(
+                            &s.value_padded(LEN).map_err(bad)?)?)
                         .map_err(bad)?;
-                    let dig = Scalar::from_bytes_reduced(
-                        GA::from_slice(&pad_truncating(digest, LEN)));
+                    let dig = pad_truncating(digest, LEN);
+                    let dig = GA::try_from_slice(&dig)?;
                     key.verify_prehashed(&dig, &sig).map_err(bad)
                 },
                 _ => Err(Error::UnsupportedEllipticCurve(curve.clone()).into()),
@@ -345,42 +375,12 @@ impl<P: key::KeyParts, R: key::KeyRole> Key<P, R> {
 impl<R> Key4<SecretParts, R>
     where R: key::KeyRole,
 {
-
-    /// Creates a new OpenPGP secret key packet for an existing X25519 key.
-    ///
-    /// The ECDH key will use hash algorithm `hash` and symmetric
-    /// algorithm `sym`.  If one or both are `None` secure defaults
-    /// will be used.  The key will have it's creation date set to
-    /// `ctime` or the current time if `None` is given.
-    pub fn import_secret_cv25519<H, S, T>(private_key: &[u8],
-                                          hash: H, sym: S, ctime: T)
-        -> Result<Self> where H: Into<Option<HashAlgorithm>>,
-                              S: Into<Option<SymmetricAlgorithm>>,
-                              T: Into<Option<SystemTime>>
+    pub(crate) fn derive_cv25519_public_key(private_key: &Protected) -> Result<[u8; 32]>
     {
         use x25519_dalek::{PublicKey, StaticSecret};
 
-        let secret = StaticSecret::from(<[u8; 32]>::try_from(private_key)?);
-        let public_key = PublicKey::from(&secret);
-
-        let mut private_key = Vec::from(private_key);
-        private_key.reverse();
-
-        use crate::crypto::ecdh;
-        Self::with_secret(
-            ctime.into().unwrap_or_else(crate::now),
-            PublicKeyAlgorithm::ECDH,
-            mpi::PublicKey::ECDH {
-                curve: Curve::Cv25519,
-                hash: hash.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kdf_hash(&Curve::Cv25519)),
-                sym: sym.into().unwrap_or_else(
-                    || ecdh::default_ecdh_kek_cipher(&Curve::Cv25519)),
-                q: MPI::new_compressed_point(&*public_key.as_bytes()),
-            },
-            mpi::SecretKeyMaterial::ECDH {
-                scalar: private_key.into(),
-            }.into())
+        let secret = StaticSecret::from(<[u8; 32]>::try_from(&private_key[..])?);
+        Ok(*PublicKey::from(&secret).as_bytes())
     }
 
     /// Creates a new OpenPGP secret key packet for an existing Ed25519 key.
@@ -412,7 +412,7 @@ impl<R> Key4<SecretParts, R>
                 q: mpi::MPI::new(&public)
             },
             mpi::SecretKeyMaterial::EdDSA {
-                scalar: mpi::MPI::new(private_key).into(),
+                scalar: private_key.into(),
             }.into()
         )
     }
@@ -452,17 +452,17 @@ impl<R> Key4<SecretParts, R>
                 n: mpi::MPI::new(&n.to_bytes_be()),
             },
             mpi::SecretKeyMaterial::RSA {
-                d: mpi::MPI::new(d).into(),
-                p: mpi::MPI::new(p).into(),
-                q: mpi::MPI::new(q).into(),
-                u: mpi::MPI::new(&u.to_bytes_be()).into(),
+                d: d.into(),
+                p: p.into(),
+                q: q.into(),
+                u: u.to_bytes_be().into(),
             }.into()
         )
     }
 
     /// Generates a new RSA key with a public modulos of size `bits`.
     pub fn generate_rsa(bits: usize) -> Result<Self> {
-        let key = RSAPrivateKey::new(&mut OsRng, bits)?;
+        let key = RsaPrivateKey::new(&mut rsa::rand_core::OsRng, bits)?;
         let (p, q) = match key.primes() {
             [p, q] => (p, q),
             _ => panic!("RSA key generation resulted in wrong number of primes"),
@@ -477,10 +477,10 @@ impl<R> Key4<SecretParts, R>
         };
 
         let private = mpi::SecretKeyMaterial::RSA {
-            p: mpi::MPI::new(&p.to_bytes_be()).into(),
-            q: mpi::MPI::new(&q.to_bytes_be()).into(),
-            d: mpi::MPI::new(&key.d().to_bytes_be()).into(),
-            u: mpi::MPI::new(&u.to_bytes_be()).into(),
+            p: p.to_bytes_be().into(),
+            q: q.to_bytes_be().into(),
+            d: key.d().to_bytes_be().into(),
+            u: u.to_bytes_be().into(),
         };
 
         Self::with_secret(
@@ -505,7 +505,12 @@ impl<R> Key4<SecretParts, R>
             (Curve::Ed25519, true) => {
                 use ed25519_dalek::Keypair;
 
-                let Keypair { public, secret } = Keypair::generate(&mut OsRng);
+                // ed25519_dalek v1.0.1 doesn't reexport OsRng.  It
+                // depends on 0.7.
+                use rand07::rngs::OsRng as OsRng;
+
+                let Keypair { public, secret }
+                    = Keypair::generate(&mut OsRng);
 
                 let secret: Protected = secret.as_bytes().as_ref().into();
 
@@ -525,7 +530,11 @@ impl<R> Key4<SecretParts, R>
             (Curve::Cv25519, false) => {
                 use x25519_dalek::{StaticSecret, PublicKey};
 
-                let private_key = StaticSecret::new(OsRng);
+                // x25519_dalek v1.1 doesn't reexport OsRng.  It
+                // depends on rand 0.8.
+                use rand::rngs::OsRng;
+
+                let private_key = StaticSecret::new(&mut OsRng);
                 let public_key = PublicKey::from(&private_key);
 
                 let mut private_key = Vec::from(private_key.to_bytes());
