@@ -310,7 +310,8 @@ impl Digest for HashDumper {
 
 impl io::Write for HashDumper {
     fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
-        self.hasher.write(buf)
+        self.update(buf);
+        Ok(buf.len())
     }
 
     fn flush(&mut self) -> io::Result<()> {
