@@ -71,10 +71,21 @@ pub mod hex {
         /// The dump is written to `inner`.  Every line is indented with
         /// `indent`.
         pub fn new<I: AsRef<str>>(inner: W, indent: I) -> Self {
+            Self::with_offset(inner, indent, 0)
+        }
+
+        /// Creates a new dumper starting at the given offset.
+        ///
+        /// The dump is written to `inner`.  Every line is indented with
+        /// `indent`.
+        pub fn with_offset<I>(inner: W, indent: I, offset: usize) -> Self
+        where
+            I: AsRef<str>,
+        {
             Dumper {
                 inner,
                 indent: indent.as_ref().into(),
-                offset: 0,
+                offset,
             }
         }
 
@@ -161,8 +172,9 @@ pub mod hex {
                 self.offset += 1;
                 match self.offset % 16 {
                     0 => {
-                        if let Some(msg) = labeler(
-                            first_label_offset, &buf[data_start..i + 1])
+                        if let Some(msg) = Some(&buf[data_start..i + 1])
+                            .filter(|b| ! b.is_empty())
+                            .and_then(|b| labeler(first_label_offset, b))
                         {
                             write!(self.inner, "   {}", msg)?;
                             // Only the first label is offset.
@@ -176,8 +188,9 @@ pub mod hex {
                 }
             }
 
-            if let Some(msg) = labeler(
-                first_label_offset, &buf[data_start..])
+            if let Some(msg) = Some(&buf[data_start..])
+                .filter(|b| ! b.is_empty())
+                .and_then(|b| labeler(first_label_offset, b))
             {
                 for i in self.offset % 16 .. 16 {
                     if i != 7 {
