@@ -336,7 +336,13 @@ pub enum Error {
     /// The optional time is the time at which the operation was
     /// determined to no longer be secure.
     #[error("{0} is not considered secure{}",
-            .1.as_ref().map(|t| format!(" since {}", crate::fmt::time(t)))
+            .1.as_ref().map(|t| {
+                if *t == std::time::UNIX_EPOCH {
+                    "".to_string()
+                } else {
+                    format!(" since {}", crate::fmt::time(t))
+                }
+            })
             .unwrap_or_else(|| "".into()))]
     PolicyViolation(String, Option<std::time::SystemTime>),
 }
@@ -374,4 +380,21 @@ mod arbitrary_helper {
             panic!()
         }
     }
+
+    pub(crate) fn arbitrary_slice<T>(g: &mut Gen, s: &mut [T])
+    where
+        T: Arbitrary,
+    {
+        s.iter_mut().for_each(|p| *p = Arbitrary::arbitrary(g));
+    }
+
+    pub(crate) fn arbitrary_bounded_vec<T>(g: &mut Gen, limit: usize) -> Vec<T>
+    where
+        T: Arbitrary + Default,
+    {
+        let mut v = vec![Default::default();
+                         gen_arbitrary_from_range(0..limit, g)];
+        arbitrary_slice(g, &mut v[..]);
+        v
+    }
 }
