@@ -458,19 +458,20 @@ impl SKESK5 {
         let key = s2k.derive_key(password, esk_algo.key_size()?)?;
         let mut iv = vec![0u8; esk_aead.nonce_size()?];
         crypto::random(&mut iv);
-        let mut ctx = esk_aead.context(esk_algo, &key, &iv, CipherOp::Encrypt)?;
+        let aad = [0xc3, 5, esk_algo.into(), esk_aead.into()];
+        let mut ctx = esk_aead.context(esk_algo, &key, &aad, &iv,
+                                       CipherOp::Encrypt)?;
 
-        // Prepare associated data.
-        let ad = [0xc3, 5, esk_algo.into(), esk_aead.into()];
-        ctx.update(&ad);
+        // Encrypt the session key with the KEK.
+        let mut esk_digest =
+            vec![0u8; session_key.len() + esk_aead.digest_size()?];
+        ctx.encrypt_seal(&mut esk_digest, session_key)?;
 
-        // We need to prefix the cipher specifier to the session key.
-        let mut esk = vec![0u8; session_key.len()];
-        ctx.encrypt(&mut esk, session_key);
-
-        // Digest.
-        let mut digest = vec![0u8; esk_aead.digest_size()?];
-        ctx.digest(&mut digest);
+        let digest = esk_digest[session_key.len()..].to_vec();
+        let esk = {
+            crate::vec_truncate(&mut esk_digest, session_key.len());
+            esk_digest
+        };
 
         SKESK5::new(esk_algo, esk_aead, s2k, iv.into_boxed_slice(), esk.into(),
                     digest.into_boxed_slice())
@@ -494,21 +495,19 @@ impl SKESK5 {
 
         if let Some(esk) = self.esk()? {
             // Use the derived key to decrypt the ESK.
+            let aad = [0xc3, 5 /* Version.  */, self.symmetric_algo().into(),
+                       self.aead_algo.into()];
             let mut cipher = self.aead_algo.context(
-                self.symmetric_algo(), &key, self.aead_iv()?, CipherOp::Decrypt)?;
+                self.symmetric_algo(), &key, &aad, self.aead_iv()?,
+                CipherOp::Decrypt)?;
 
-            let ad = [0xc3, 5 /* Version.  */, self.symmetric_algo().into(),
-                      self.aead_algo.into()];
-            cipher.update(&ad);
             let mut plain: SessionKey = vec![0; esk.len()].into();
-            let mut digest = vec![0; self.aead_algo.digest_size()?];
-            cipher.decrypt(&mut plain, esk);
-            cipher.digest(&mut digest);
-            if digest[..] == self.aead_digest[..] {
-                Ok((SymmetricAlgorithm::Unencrypted, plain))
-            } else {
-                Err(Error::ManipulatedMessage.into())
-            }
+            let mut chunk =
+                Vec::with_capacity(esk.len() + self.aead_digest.len());
+            chunk.extend_from_slice(esk);
+            chunk.extend_from_slice(&self.aead_digest);
+            cipher.decrypt_verify(&mut plain, &chunk)?;
+            Ok((SymmetricAlgorithm::Unencrypted, plain))
         } else {
             Err(Error::MalformedPacket(
                 "No encrypted session key in v5 SKESK packet".into())
@@ -572,7 +571,7 @@ impl From<SKESK5> for Packet {
 #[cfg(test)]
 impl Arbitrary for SKESK5 {
     fn arbitrary(g: &mut Gen) -> Self {
-        let algo = AEADAlgorithm::EAX;  // The only one we dig.
+        let algo = AEADAlgorithm::const_default();
         let mut iv = vec![0u8; algo.nonce_size().unwrap()];
         for b in iv.iter_mut() {
             *b = u8::arbitrary(g);
@@ -639,9 +638,11 @@ mod test {
                        &[0xb2, 0x55, 0x69, 0xb9, 0x54, 0x32, 0x45, 0x66,
                          0x45, 0x27, 0xc4, 0x97, 0x6e, 0x7a, 0x5d, 0x6e][..]);
 
-            assert_eq!(&s.decrypt(&password).unwrap().1[..],
+            if AEADAlgorithm::EAX.is_supported() {
+                assert_eq!(&s.decrypt(&password).unwrap().1[..],
                        &[0x86, 0xf1, 0xef, 0xb8, 0x69, 0x52, 0x32, 0x9f,
                          0x24, 0xac, 0xd3, 0xbf, 0xd0, 0xe5, 0x34, 0x6d][..]);
+            }
         } else {
             panic!("bad packet");
         }
