@@ -72,6 +72,16 @@ impl<T> HashingMode<T> {
         }
     }
 
+    pub(crate) fn mapf<U, F: Fn(T) -> Result<U>>(self, f: F)
+                                                 -> Result<HashingMode<U>> {
+        use self::HashingMode::*;
+        match self {
+            Binary(t) => Ok(Binary(f(t)?)),
+            Text(t) => Ok(Text(f(t)?)),
+            TextLastWasCr(t) => Ok(TextLastWasCr(f(t)?)),
+        }
+    }
+
     pub(crate) fn as_ref(&self) -> &T {
         use self::HashingMode::*;
         match self {
@@ -208,18 +218,24 @@ impl<R: BufferedReader<Cookie>> HashedReader<R> {
     /// compute the hash.
     pub fn new(reader: R, hashes_for: HashesFor,
                algos: Vec<HashingMode<HashAlgorithm>>)
-            -> Self {
+            -> Result<Self> {
         let mut cookie = Cookie::default();
-        for mode in &algos {
-            cookie.sig_group_mut().hashes
-                .push(mode.map(|algo| algo.context().unwrap())); // XXX: Don't unwrap.
+
+        for mode in algos {
+            let mode = mode.mapf(|algo| {
+                let ctx = algo.context()?;
+                Ok(ctx)
+            })?;
+
+            cookie.sig_group_mut().hashes.push(mode);
         }
+
         cookie.hashes_for = hashes_for;
 
-        HashedReader {
+        Ok(HashedReader {
             reader,
             cookie,
-        }
+        })
     }
 }
 
@@ -290,8 +306,12 @@ impl Cookie {
         let ngroups = self.sig_groups.len();
 
         assert_eq!(self.hashes_for, HashesFor::CleartextSignature);
-        // There is exactly one group.
-        assert_eq!(ngroups, 1);
+        // There is exactly one group.  However, this can momentarily
+        // be violated if there are One-Pass-Signature packets in the
+        // signature block.  This doesn't last long though: the
+        // message parser will reject the message because it doesn't
+        // adhere to the grammar.
+        assert!(ngroups == 1 || ngroups == /* momentarily */ 2);
 
         tracer!(TRACE, "Cookie::hash_update_csf", level);
         t!("Cleartext Signature Framework message");
@@ -325,7 +345,8 @@ impl Cookie {
                 t!("{:?}: {:?} hashing {} stashed bytes.",
                    hashes_for, h.map(|ctx| ctx.algo()),
                    stashed_data.len());
-                assert!(matches!(h, HashingMode::Text(_)),
+                assert!(matches!(h, HashingMode::Text(_)
+                                 | HashingMode::TextLastWasCr(_)),
                         "CSF transformation uses text signatures");
                 h.update(&stashed_data[..]);
             }
@@ -350,7 +371,8 @@ impl Cookie {
         for h in self.sig_groups[0].hashes.iter_mut() {
             t!("{:?}: {:?} hashing {} bytes.",
                hashes_for, h.map(|ctx| ctx.algo()), l);
-            assert!(matches!(h, HashingMode::Text(_)),
+            assert!(matches!(h, HashingMode::Text(_)
+                             | HashingMode::TextLastWasCr(_)),
                     "CSF transformation uses text signatures");
             h.update(&data[..l]);
         }
@@ -489,7 +511,7 @@ pub(crate) fn hash_buffered_reader<R>(reader: R,
     where R: BufferedReader<crate::parse::Cookie>,
 {
     let mut reader
-        = HashedReader::new(reader, HashesFor::Signature, algos.to_vec());
+        = HashedReader::new(reader, HashesFor::Signature, algos.to_vec())?;
 
     // Hash all of the data.
     reader.drop_eof()?;
@@ -546,7 +568,7 @@ mod test {
                 = HashedReader::new(reader, HashesFor::MDC,
                                     test.expected.keys().cloned()
                                     .map(HashingMode::Binary)
-                                    .collect());
+                                    .collect()).unwrap();
 
             assert_eq!(reader.steal_eof().unwrap(), test.data);
 
