@@ -10,7 +10,7 @@ use crate::{
     HashAlgorithm,
 };
 use crate::types::Curve;
-use crate::crypto::mpi::{self, MPI};
+use crate::crypto::mpi::{self, MPI, ProtectedMPI};
 use crate::parse::{
     PacketHeaderParser,
     Cookie,
@@ -26,7 +26,7 @@ impl mpi::PublicKey {
     {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(bio);
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
         Self::_parse(algo, &mut php)
     }
 
@@ -35,9 +35,9 @@ impl mpi::PublicKey {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+    pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
-        php: &mut PacketHeaderParser<T>)
+        php: &mut PacketHeaderParser<'_>)
         -> Result<Self>
     {
         use crate::PublicKeyAlgorithm::*;
@@ -151,13 +151,29 @@ impl mpi::SecretKeyMaterial {
     /// Parses secret key MPIs for `algo` plus their SHA1 checksum.
     ///
     /// Fails if the checksum is wrong.
+    #[deprecated(
+        since = "1.14.0",
+        note = "Leaks secrets into the heap, use [`SecretKeyMaterial::from_bytes_with_checksum`]")]
     pub fn parse_with_checksum<R: Read + Send + Sync>(algo: PublicKeyAlgorithm,
                                         reader: R,
                                         checksum: mpi::SecretKeyChecksum)
                                         -> Result<Self> {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(bio);
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
+        Self::_parse(algo, &mut php, Some(checksum))
+    }
+
+    /// Parses secret key MPIs for `algo` plus their SHA1 checksum.
+    ///
+    /// Fails if the checksum is wrong.
+    pub fn from_bytes_with_checksum(algo: PublicKeyAlgorithm,
+                                    bytes: &[u8],
+                                    checksum: mpi::SecretKeyChecksum)
+                                    -> Result<Self> {
+        let bio = buffered_reader::Memory::with_cookie(
+            bytes, Cookie::default());
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
         Self::_parse(algo, &mut php, Some(checksum))
     }
 
@@ -166,11 +182,14 @@ impl mpi::SecretKeyMaterial {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    #[deprecated(
+        since = "1.14.0",
+        note = "Leaks secrets into the heap, use [`SecretKeyMaterial::from_bytes`]")]
     pub fn parse<R: Read + Send + Sync>(algo: PublicKeyAlgorithm, reader: R) -> Result<Self>
     {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(bio);
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
         Self::_parse(algo, &mut php, None)
     }
 
@@ -179,9 +198,21 @@ impl mpi::SecretKeyMaterial {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+    pub fn from_bytes(algo: PublicKeyAlgorithm, buf: &[u8]) -> Result<Self> {
+        let bio = buffered_reader::Memory::with_cookie(
+            buf, Cookie::default());
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
+        Self::_parse(algo, &mut php, None)
+    }
+
+    /// Parses a set of OpenPGP MPIs representing a secret key.
+    ///
+    /// See [Section 3.2 of RFC 4880] for details.
+    ///
+    ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
+    pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
-        php: &mut PacketHeaderParser<T>,
+        php: &mut PacketHeaderParser<'_>,
         checksum: Option<mpi::SecretKeyChecksum>,
     )
         -> Result<Self>
@@ -191,62 +222,58 @@ impl mpi::SecretKeyMaterial {
         #[allow(deprecated)]
         let mpis: Result<Self> = match algo {
             RSAEncryptSign | RSAEncrypt | RSASign => {
-                let d = MPI::parse("rsa_secret_d_len", "rsa_secret_d", php)?;
-                let p = MPI::parse("rsa_secret_p_len", "rsa_secret_p", php)?;
-                let q = MPI::parse("rsa_secret_q_len", "rsa_secret_q", php)?;
-                let u = MPI::parse("rsa_secret_u_len", "rsa_secret_u", php)?;
-
                 Ok(mpi::SecretKeyMaterial::RSA {
-                    d: d.into(),
-                    p: p.into(),
-                    q: q.into(),
-                    u: u.into(),
+                    d: ProtectedMPI::parse(
+                        "rsa_secret_d_len", "rsa_secret_d", php)?,
+                    p: ProtectedMPI::parse(
+                        "rsa_secret_p_len", "rsa_secret_p", php)?,
+                    q: ProtectedMPI::parse(
+                        "rsa_secret_q_len", "rsa_secret_q", php)?,
+                    u: ProtectedMPI::parse(
+                        "rsa_secret_u_len", "rsa_secret_u", php)?,
                 })
             }
 
             DSA => {
-                let x = MPI::parse("dsa_secret_len", "dsa_secret", php)?;
-
                 Ok(mpi::SecretKeyMaterial::DSA {
-                    x: x.into(),
+                    x: ProtectedMPI::parse(
+                        "dsa_secret_len", "dsa_secret", php)?,
                 })
             }
 
             ElGamalEncrypt | ElGamalEncryptSign => {
-                let x = MPI::parse("elgamal_secret_len", "elgamal_secret",
-                                   php)?;
-
                 Ok(mpi::SecretKeyMaterial::ElGamal {
-                    x: x.into(),
+                    x: ProtectedMPI::parse(
+                        "elgamal_secret_len", "elgamal_secret", php)?,
                 })
             }
 
             EdDSA => {
                 Ok(mpi::SecretKeyMaterial::EdDSA {
-                    scalar: MPI::parse("eddsa_secret_len", "eddsa_secret", php)?
-                                .into()
+                    scalar: ProtectedMPI::parse(
+                        "eddsa_secret_len", "eddsa_secret", php)?,
                 })
             }
 
             ECDSA => {
                 Ok(mpi::SecretKeyMaterial::ECDSA {
-                    scalar: MPI::parse("ecdsa_secret_len", "ecdsa_secret", php)?
-                                .into()
+                    scalar: ProtectedMPI::parse(
+                        "ecdsa_secret_len", "ecdsa_secret", php)?,
                 })
             }
 
             ECDH => {
                 Ok(mpi::SecretKeyMaterial::ECDH {
-                    scalar: MPI::parse("ecdh_secret_len", "ecdh_secret", php)?
-                                .into()
+                    scalar: ProtectedMPI::parse(
+                        "ecdh_secret_len", "ecdh_secret", php)?,
                 })
             }
 
             Unknown(_) | Private(_) => {
                 let mut mpis = Vec::new();
-                while let Ok(mpi) = MPI::parse("unknown_len",
+                while let Ok(mpi) = ProtectedMPI::parse("unknown_len",
                                                "unknown", php) {
-                    mpis.push(mpi.into());
+                    mpis.push(mpi);
                 }
                 let rest = php.parse_bytes_eof("rest")?;
 
@@ -309,7 +336,7 @@ impl mpi::Ciphertext {
     {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(bio);
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
         Self::_parse(algo, &mut php)
     }
 
@@ -319,9 +346,9 @@ impl mpi::Ciphertext {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+    pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
-        php: &mut PacketHeaderParser<T>)
+        php: &mut PacketHeaderParser<'_>)
         -> Result<Self> {
         use crate::PublicKeyAlgorithm::*;
 
@@ -388,7 +415,7 @@ impl mpi::Signature {
     {
         let bio = buffered_reader::Generic::with_cookie(
             reader, None, Cookie::default());
-        let mut php = PacketHeaderParser::new_naked(bio);
+        let mut php = PacketHeaderParser::new_naked(bio.as_boxed());
         Self::_parse(algo, &mut php)
     }
 
@@ -398,9 +425,9 @@ impl mpi::Signature {
     /// See [Section 3.2 of RFC 4880] for details.
     ///
     ///   [Section 3.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-3.2
-    pub(crate) fn _parse<'a, T: 'a + BufferedReader<Cookie>>(
+    pub(crate) fn _parse(
         algo: PublicKeyAlgorithm,
-        php: &mut PacketHeaderParser<T>)
+        php: &mut PacketHeaderParser<'_>)
         -> Result<Self> {
         use crate::PublicKeyAlgorithm::*;
 
