@@ -5,6 +5,7 @@ use crate::seal;
 use crate::serialize::{
     PacketRef,
     Marshal, MarshalInto,
+    NetLength,
     generic_serialize_into, generic_export_into,
 };
 
@@ -196,8 +197,8 @@ impl Cert {
     ///
     /// This object writes out secret keys during serialization.
     ///
-    /// [`TSK`]: ../serialize/struct.TSK.html
-    pub fn as_tsk<'a>(&'a self) -> TSK<'a> {
+    /// [`TSK`]: crate::serialize::TSK
+    pub fn as_tsk(&self) -> TSK {
         TSK::new(self)
     }
 }
@@ -209,8 +210,8 @@ impl Cert {
 /// use [`Cert::as_tsk()`] to create a `TSK`, which is a shim on top
 /// of the `Cert`, and serialize this.
 ///
-/// [`Cert`]: ../cert/struct.Cert.html
-/// [`Cert::as_tsk()`]: ../cert/struct.Cert.html#method.as_tsk
+/// [`Cert`]: crate::cert::Cert
+/// [`Cert::as_tsk()`]: crate::cert::Cert::as_tsk()
 ///
 /// # Examples
 ///
@@ -230,16 +231,51 @@ impl Cert {
 /// ```
 pub struct TSK<'a> {
     pub(crate) cert: &'a Cert,
-    filter: Option<Box<dyn Fn(&'a key::UnspecifiedSecret) -> bool + 'a>>,
+    filter: Box<dyn Fn(&'a key::UnspecifiedSecret) -> bool + 'a>,
     emit_stubs: bool,
 }
 
+impl PartialEq for TSK<'_> {
+    fn eq(&self, other: &Self) -> bool {
+        // First, compare the certs.  If they are not equal, then the
+        // TSK's cannot possibly be equal.
+        if self.cert != other.cert {
+            return false;
+        }
+
+        // Second, consider all the keys.
+        for (a, b) in self.cert.keys()
+            .zip(other.cert.keys())
+        {
+            match (a.has_secret()
+                   && (self.filter)(a.key().parts_as_secret().expect("has secret")),
+                   b.has_secret()
+                   && (other.filter)(b.key().parts_as_secret().expect("has_secret")))
+            {
+                // Both have secrets.  Compare secrets.
+                (true, true) => if a.optional_secret() != b.optional_secret() {
+                    return false;
+                },
+                // No secrets.  Equal iff both or neither emit stubs.
+                (false, false) => if self.emit_stubs != other.emit_stubs {
+                    return false;
+                },
+                // Otherwise, they differ.
+                _ => return false,
+            }
+        }
+
+        // Everything matched.
+        true
+    }
+}
+
 impl<'a> TSK<'a> {
     /// Creates a new view for the given `Cert`.
     fn new(cert: &'a Cert) -> Self {
         Self {
             cert,
-            filter: None,
+            filter: Box::new(|_| true),
             emit_stubs: false,
         }
     }
@@ -277,7 +313,7 @@ impl<'a> TSK<'a> {
     pub fn set_filter<P>(mut self, predicate: P) -> Self
         where P: 'a + Fn(&'a key::UnspecifiedSecret) -> bool
     {
-        self.filter = Some(Box::new(predicate));
+        self.filter = Box::new(predicate);
         self
     }
 
@@ -300,9 +336,9 @@ impl<'a> TSK<'a> {
     ///
     /// See [this test] for support in other implementations.
     ///
-    ///   [`TSK::set_filter`]: #method.set_filter
+    ///   [`TSK::set_filter`]: TSK::set_filter()
     ///   [Section 11.2 of RFC 4880]: https://tools.ietf.org/html/rfc4880#section-11.2
-    ///   [`S2K`]: ../crypto/enum.S2K.html
+    ///   [`S2K`]: super::crypto::S2K
     ///   [this test]: https://tests.sequoia-pgp.org/#Detached_primary_key
     ///
     /// # Examples
@@ -385,8 +421,7 @@ impl<'a> TSK<'a> {
             |o: &mut dyn std::io::Write, key: &'a key::UnspecifiedSecret,
              tag_public, tag_secret|
         {
-            let tag = if key.has_secret()
-                && self.filter.as_ref().map(|f| f(key)).unwrap_or(true) {
+            let tag = if key.has_secret() && (self.filter)(key) {
                 tag_secret
             } else {
                 tag_public
@@ -497,7 +532,7 @@ impl<'a> TSK<'a> {
                 continue;
             }
 
-            PacketRef::Unknown(&u.unknown()).serialize(o)?;
+            PacketRef::Unknown(u.unknown()).serialize(o)?;
 
             for s in u.signatures() {
                 serialize_sig(o, s)?;
@@ -535,8 +570,7 @@ impl<'a> MarshalInto for TSK<'a> {
         let serialized_len_key
             = |key: &'a key::UnspecifiedSecret, tag_public, tag_secret|
         {
-            let tag = if key.has_secret()
-                && self.filter.as_ref().map(|f| f(key)).unwrap_or(true) {
+            let tag = if key.has_secret() && (self.filter)(key) {
                 tag_secret
             } else {
                 tag_public
@@ -546,7 +580,7 @@ impl<'a> MarshalInto for TSK<'a> {
                                    || tag == Tag::PublicSubkey) {
                 // Emit a GnuPG-style secret key stub.  The stub
                 // extends the public key by 8 bytes.
-                let l = key.net_len_key(false) + 8;
+                let l = key.parts_as_public().net_len() + 8;
                 return 1 // CTB
                     + BodyLength::Full(l as u32).serialized_len()
                     + l;
@@ -754,11 +788,11 @@ mod test {
         ]).unwrap();
 
         assert_eq!(cert.subkeys().count(), 1);
-        cert.subkeys().nth(0).unwrap().binding_signature(p, None).unwrap();
+        cert.subkeys().next().unwrap().binding_signature(p, None).unwrap();
         assert_eq!(cert.userids().count(), 1);
-        assert!(cert.userids().with_policy(p, None).nth(0).is_some());
+        assert!(cert.userids().with_policy(p, None).next().is_some());
         assert_eq!(cert.user_attributes().count(), 1);
-        assert!(cert.user_attributes().with_policy(p, None).nth(0).is_some());
+        assert!(cert.user_attributes().with_policy(p, None).next().is_some());
 
         // The binding signature is not exportable, so when we export
         // and re-parse, we expect the userid to be gone.
@@ -870,4 +904,67 @@ mod test {
         assert_eq!(buf, buf_);
         Ok(())
     }
+
+    /// Checks partial equality of certificates and TSKs.
+    #[test]
+    fn issue_701() -> Result<()> {
+        let cert_0 = Cert::from_bytes(crate::tests::key("testy.pgp"))?;
+        let cert_1 = Cert::from_bytes(crate::tests::key("testy.pgp"))?;
+        let tsk_0 = Cert::from_bytes(crate::tests::key("testy-private.pgp"))?;
+        let tsk_1 = Cert::from_bytes(crate::tests::key("testy-private.pgp"))?;
+
+        // We define equality by equality of the serialized form.
+        // This macro checks that.
+        macro_rules! check {
+            ($a: expr, $b: expr, $expectation: expr) => {{
+                let a = $a;
+                let b = $b;
+                let serialized_eq = a.to_vec()? == b.to_vec()?;
+                let eq = a == b;
+                assert_eq!(serialized_eq, eq);
+                assert_eq!(eq, $expectation);
+            }};
+        }
+
+        // Equal as certs, because equality is defined by equality of
+        // the serialized form, and serializing a cert never writes
+        // out any secrets.
+        check!(&cert_0, &cert_1, true);
+        check!(&cert_0, &tsk_0, true);
+
+        // Filters out secrets.
+        let no_secrets = |_| false;
+
+        // TSK's equality.
+        check!(tsk_0.as_tsk(), tsk_1.as_tsk(), true);
+
+        // Without secrets.
+        check!(tsk_0.as_tsk().set_filter(no_secrets),
+               tsk_1.as_tsk().set_filter(no_secrets),
+               true);
+
+        // Still equal if one uses stubs and the other one does not if
+        // every key has a secret, i.e. stubs are not in fact used.
+        check!(
+            tsk_0.as_tsk().emit_secret_key_stubs(true),
+            tsk_1.as_tsk(),
+            true);
+
+        // No longer equal if one actually emits stubs.
+        check!(
+            tsk_0.as_tsk().emit_secret_key_stubs(true).set_filter(no_secrets),
+            tsk_1.as_tsk(),
+            false);
+
+        // Certs are not equal to TSKs, because here the secrets are
+        // written out when serialized.
+        check!(cert_0.as_tsk(), tsk_0.as_tsk(), false);
+
+        // Equal, if we filter out the secrets.
+        check!(cert_0.as_tsk(),
+               tsk_0.as_tsk().set_filter(no_secrets),
+               true);
+
+        Ok(())
+    }
 }
