@@ -128,7 +128,7 @@ pub enum PublicKeyAlgorithm {
 assert_send_and_sync!(PublicKeyAlgorithm);
 
 #[allow(deprecated)]
-const PUBLIC_KEY_ALGORITHM_VARIANTS: [PublicKeyAlgorithm; 9] = [
+pub(crate) const PUBLIC_KEY_ALGORITHM_VARIANTS: [PublicKeyAlgorithm; 9] = [
     PublicKeyAlgorithm::RSAEncryptSign,
     PublicKeyAlgorithm::RSAEncrypt,
     PublicKeyAlgorithm::RSASign,
@@ -204,7 +204,6 @@ impl PublicKeyAlgorithm {
     ///
     /// assert!(PublicKeyAlgorithm::EdDSA.is_supported());
     /// assert!(PublicKeyAlgorithm::RSAEncryptSign.is_supported());
-    /// assert!(!PublicKeyAlgorithm::ElGamalEncrypt.is_supported());
     /// assert!(!PublicKeyAlgorithm::Private(101).is_supported());
     /// ```
     pub fn is_supported(&self) -> bool {
@@ -366,6 +365,13 @@ pub enum Curve {
     /// Unknown curve.
     Unknown(Box<[u8]>),
 }
+impl Curve {
+    /// Hack!  Curve is not non-exhaustive, so we cannot easily add
+    /// a variant.
+    pub(crate) fn is_brainpoolp384(&self) -> bool {
+        self.oid() == BRAINPOOL_P384_OID
+    }
+}
 assert_send_and_sync!(Curve);
 
 impl Curve {
@@ -399,6 +405,7 @@ impl Curve {
             NistP384 => Some(384),
             NistP521 => Some(521),
             BrainpoolP256 => Some(256),
+            Unknown(_) if self.is_brainpoolp384() => Some(384),
             BrainpoolP512 => Some(512),
             Ed25519 => Some(256),
             Cv25519 => Some(256),
@@ -456,6 +463,8 @@ impl fmt::Display for Curve {
                 NistP384 => f.write_str("NIST curve P-384"),
                 NistP521 => f.write_str("NIST curve P-521"),
                 BrainpoolP256 => f.write_str("brainpoolP256r1"),
+                Unknown(_) if self.is_brainpoolp384() =>
+                    f.write_str("brainpoolP384r1"),
                 BrainpoolP512 => f.write_str("brainpoolP512r1"),
                 Ed25519
                     => f.write_str("D.J. Bernstein's \"Twisted\" Edwards curve Ed25519"),
@@ -470,6 +479,8 @@ impl fmt::Display for Curve {
                 NistP384 => f.write_str("NIST P-384"),
                 NistP521 => f.write_str("NIST P-521"),
                 BrainpoolP256 => f.write_str("brainpoolP256r1"),
+                Unknown(_) if self.is_brainpoolp384() =>
+                    f.write_str("brainpoolP384r1"),
                 BrainpoolP512 => f.write_str("brainpoolP512r1"),
                 Ed25519
                     => f.write_str("Ed25519"),
@@ -487,6 +498,8 @@ const NIST_P384_OID: &[u8] = &[0x2B, 0x81, 0x04, 0x00, 0x22];
 const NIST_P521_OID: &[u8] = &[0x2B, 0x81, 0x04, 0x00, 0x23];
 const BRAINPOOL_P256_OID: &[u8] =
     &[0x2B, 0x24, 0x03, 0x03, 0x02, 0x08, 0x01, 0x01, 0x07];
+const BRAINPOOL_P384_OID: &[u8] =
+    &[0x2B, 0x24, 0x03, 0x03, 0x02, 0x08, 0x01, 0x01, 0x0B];
 const BRAINPOOL_P512_OID: &[u8] =
     &[0x2B, 0x24, 0x03, 0x03, 0x02, 0x08, 0x01, 0x01, 0x0D];
 const ED25519_OID: &[u8] =
@@ -514,6 +527,7 @@ impl Curve {
             NIST_P384_OID => Curve::NistP384,
             NIST_P521_OID => Curve::NistP521,
             BRAINPOOL_P256_OID => Curve::BrainpoolP256,
+            BRAINPOOL_P384_OID => Curve::Unknown(BRAINPOOL_P384_OID.into()),
             BRAINPOOL_P512_OID => Curve::BrainpoolP512,
             ED25519_OID => Curve::Ed25519,
             CV25519_OID => Curve::Cv25519,
@@ -569,6 +583,7 @@ impl Curve {
             Curve::NistP384 => Ok(384),
             Curve::NistP521 => Ok(521),
             Curve::BrainpoolP256 => Ok(256),
+            Curve::Unknown(_) if self.is_brainpoolp384() => Ok(384),
             Curve::BrainpoolP512 => Ok(512),
             Curve::Ed25519 => Ok(256),
             Curve::Cv25519 => Ok(256),
@@ -597,15 +612,16 @@ impl Curve {
 #[cfg(test)]
 impl Arbitrary for Curve {
     fn arbitrary(g: &mut Gen) -> Self {
-        match u8::arbitrary(g) % 8 {
+        match u8::arbitrary(g) % 9 {
             0 => Curve::NistP256,
             1 => Curve::NistP384,
             2 => Curve::NistP521,
             3 => Curve::BrainpoolP256,
-            4 => Curve::BrainpoolP512,
-            5 => Curve::Ed25519,
-            6 => Curve::Cv25519,
-            7 => Curve::Unknown({
+            4 => Curve::Unknown(BRAINPOOL_P384_OID.into()),
+            5 => Curve::BrainpoolP512,
+            6 => Curve::Ed25519,
+            7 => Curve::Cv25519,
+            8 => Curve::Unknown({
                 let mut k = <Vec<u8>>::arbitrary(g);
                 k.truncate(255);
                 k.into_boxed_slice()
@@ -846,6 +862,67 @@ impl SymmetricAlgorithm {
     pub fn variants() -> impl Iterator<Item=Self> {
         SYMMETRIC_ALGORITHM_VARIANTS.iter().cloned()
     }
+
+    /// Returns whether this algorithm is supported by the crypto backend.
+    ///
+    /// All backends support all the AES variants.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use sequoia_openpgp as openpgp;
+    /// use openpgp::types::SymmetricAlgorithm;
+    ///
+    /// assert!(SymmetricAlgorithm::AES256.is_supported());
+    /// assert!(SymmetricAlgorithm::TripleDES.is_supported());
+    ///
+    /// assert!(!SymmetricAlgorithm::Unencrypted.is_supported());
+    /// assert!(!SymmetricAlgorithm::Private(101).is_supported());
+    /// ```
+    pub fn is_supported(&self) -> bool {
+        self.is_supported_by_backend()
+    }
+
+    /// Length of a key for this algorithm in bytes.
+    ///
+    /// Fails if the algorithm isn't known to Sequoia.
+    pub fn key_size(self) -> Result<usize> {
+        match self {
+            SymmetricAlgorithm::IDEA => Ok(16),
+            SymmetricAlgorithm::TripleDES => Ok(24),
+            SymmetricAlgorithm::CAST5 => Ok(16),
+            // RFC4880, Section 9.2: Blowfish (128 bit key, 16 rounds)
+            SymmetricAlgorithm::Blowfish => Ok(16),
+            SymmetricAlgorithm::AES128 => Ok(16),
+            SymmetricAlgorithm::AES192 => Ok(24),
+            SymmetricAlgorithm::AES256 => Ok(32),
+            SymmetricAlgorithm::Twofish => Ok(32),
+            SymmetricAlgorithm::Camellia128 => Ok(16),
+            SymmetricAlgorithm::Camellia192 => Ok(24),
+            SymmetricAlgorithm::Camellia256 => Ok(32),
+            _ => Err(Error::UnsupportedSymmetricAlgorithm(self).into()),
+        }
+    }
+
+    /// Length of a block for this algorithm in bytes.
+    ///
+    /// Fails if the algorithm isn't known to Sequoia.
+    pub fn block_size(self) -> Result<usize> {
+        match self {
+            SymmetricAlgorithm::IDEA => Ok(8),
+            SymmetricAlgorithm::TripleDES => Ok(8),
+            SymmetricAlgorithm::CAST5 => Ok(8),
+            SymmetricAlgorithm::Blowfish => Ok(8),
+            SymmetricAlgorithm::AES128 => Ok(16),
+            SymmetricAlgorithm::AES192 => Ok(16),
+            SymmetricAlgorithm::AES256 => Ok(16),
+            SymmetricAlgorithm::Twofish => Ok(16),
+            SymmetricAlgorithm::Camellia128 => Ok(16),
+            SymmetricAlgorithm::Camellia192 => Ok(16),
+            SymmetricAlgorithm::Camellia256 => Ok(16),
+            _ => Err(Error::UnsupportedSymmetricAlgorithm(self).into()),
+        }
+    }
 }
 
 /// The AEAD algorithms as defined in [Section 9.6 of RFC 4880bis].
@@ -1701,8 +1778,8 @@ impl Arbitrary for ReasonForRevocation {
 /// or soft.
 ///
 /// A hard revocation is a revocation that indicates that the key was
-/// somehow compromised, and the provence of *all* artifacts should be
-/// called into question.
+/// somehow compromised, and the provenance of *all* artifacts should
+/// be called into question.
 ///
 /// A soft revocation is a revocation that indicates that the key
 /// should be considered invalid *after* the revocation signature's
